WinAnsiEncoding = (
                None, None, None, None, None, None, None, None, None, None, None, None,
                None, None, None, None, None, None, None, None, None, None, None, None,
                None, None, None, None, None, None, None, None, 'space', 'exclam',
                'quotedbl', 'numbersign', 'dollar', 'percent', 'ampersand',
                'quotesingle', 'parenleft', 'parenright', 'asterisk', 'plus', 'comma',
                'hyphen', 'period', 'slash', 'zero', 'one', 'two', 'three', 'four',
                'five', 'six', 'seven', 'eight', 'nine', 'colon', 'semicolon', 'less',
                'equal', 'greater', 'question', 'at', 'A', 'B', 'C', 'D', 'E', 'F',
                'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
                'U', 'V', 'W', 'X', 'Y', 'Z', 'bracketleft', 'backslash', 'bracketright',
                'asciicircum', 'underscore', 'grave', 'a', 'b', 'c', 'd', 'e', 'f',
                'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
                'u', 'v', 'w', 'x', 'y', 'z', 'braceleft', 'bar', 'braceright',
                'asciitilde', 'bullet', 'Euro', 'bullet', 'quotesinglbase', 'florin',
                'quotedblbase', 'ellipsis', 'dagger', 'daggerdbl', 'circumflex',
                'perthousand', 'Scaron', 'guilsinglleft', 'OE', 'bullet', 'Zcaron',
                'bullet', 'bullet', 'quoteleft', 'quoteright', 'quotedblleft',
                'quotedblright', 'bullet', 'endash', 'emdash', 'tilde', 'trademark',
                'scaron', 'guilsinglright', 'oe', 'bullet', 'zcaron', 'Ydieresis',
                'space', 'exclamdown', 'cent', 'sterling', 'currency', 'yen', 'brokenbar',
                'section', 'dieresis', 'copyright', 'ordfeminine', 'guillemotleft',
                'logicalnot', 'hyphen', 'registered', 'macron', 'degree', 'plusminus',
                'twosuperior', 'threesuperior', 'acute', 'mu', 'paragraph', 'periodcentered',
                'cedilla', 'onesuperior', 'ordmasculine', 'guillemotright', 'onequarter',
                'onehalf', 'threequarters', 'questiondown', 'Agrave', 'Aacute',
                'Acircumflex', 'Atilde', 'Adieresis', 'Aring', 'AE', 'Ccedilla',
                'Egrave', 'Eacute', 'Ecircumflex', 'Edieresis', 'Igrave', 'Iacute',
                'Icircumflex', 'Idieresis', 'Eth', 'Ntilde', 'Ograve', 'Oacute',
                'Ocircumflex', 'Otilde', 'Odieresis', 'multiply', 'Oslash', 'Ugrave',
                'Uacute', 'Ucircumflex', 'Udieresis', 'Yacute', 'Thorn', 'germandbls',
                'agrave', 'aacute', 'acircumflex', 'atilde', 'adieresis', 'aring', 'ae',
                'ccedilla', 'egrave', 'eacute', 'ecircumflex', 'edieresis', 'igrave',
                'iacute', 'icircumflex', 'idieresis', 'eth', 'ntilde', 'ograve', 'oacute',
                'ocircumflex', 'otilde', 'odieresis', 'divide', 'oslash', 'ugrave', 'uacute',
                'ucircumflex', 'udieresis', 'yacute', 'thorn', 'ydieresis')




#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
#history https://bitbucket.org/rptlab/reportlab/history-node/tip/src/reportlab/platypus/xpreformatted.py
__version__='3.3.0'
__doc__='''A 'rich preformatted text' widget allowing internal markup'''
from reportlab.lib import PyFontify
from reportlab.platypus.paragraph import Paragraph, cleanBlockQuotedText, _handleBulletWidth, \
     ParaLines, _getFragWords, stringWidth, getAscentDescent, imgVRange, imgNormV
from reportlab.lib.utils import isSeq
from reportlab.platypus.flowables import _dedenter

def _getFragLines(frags):
    lines = []
    cline = []
    W = frags[:]
    while W != []:
        w = W[0]
        t = w.text
        del W[0]
        i = t.find('\n')
        if i>=0:
            tleft = t[i+1:]
            cline.append(w.clone(text=t[:i]))
            lines.append(cline)
            cline = []
            if tleft!='':
                W.insert(0,w.clone(text=tleft))
        else:
            cline.append(w)
    if cline!=[]:
        lines.append(cline)
    return lines

def _split_blPara(blPara,start,stop):
    f = blPara.clone()
    for a in ('lines', 'text'):
        if hasattr(f,a): delattr(f,a)
    f.lines = blPara.lines[start:stop]
    return [f]

# Will be removed shortly.
def _countSpaces(text):
    return text.count(' ')
##  i = 0
##  s = 0
##  while 1:
##      j = text.find(' ',i)
##      if j<0: return s
##      s = s + 1
##      i = j + 1

def _getFragWord(frags,maxWidth):
    ''' given a fragment list return a list of lists
        [size, spaces, (f00,w00), ..., (f0n,w0n)]
        each pair f,w represents a style and some string
    '''
    W = []
    n = 0
    s = 0
    for f in frags:
        text = f.text[:]
        W.append((f,text))
        cb = getattr(f,'cbDefn',None)
        if cb:
            _w = getattr(cb,'width',0)
            if hasattr(_w,'normalizedValue'):
                _w._normalizer = maxWidth
        n += stringWidth(text, f.fontName, f.fontSize)

        #s = s + _countSpaces(text)
        s += text.count(' ') # much faster for many blanks

        #del f.text # we can't do this until we sort out splitting
                    # of paragraphs
    return n, s, W

class XPreformatted(Paragraph):
    def __init__(self, text, style, bulletText = None, frags=None, caseSensitive=1, dedent=0):
        self.caseSensitive = caseSensitive
        cleaner = lambda text, dedent=dedent: '\n'.join(_dedenter(text or '',dedent))
        self._setup(text, style, bulletText, frags, cleaner)

    def breakLines(self, width):
        """
        Returns a broken line structure. There are two cases

        A) For the simple case of a single formatting input fragment the output is
            A fragment specifier with
                - kind = 0
                - fontName, fontSize, leading, textColor
                - lines=  A list of lines
                
                    Each line has two items:
                    
                    1. unused width in points
                    2. a list of words

        B) When there is more than one input formatting fragment the out put is
            A fragment specifier with
                - kind = 1
                - lines =  A list of fragments each having fields:
                
                    - extraspace (needed for justified)
                    - fontSize
                    - words=word list
                    - each word is itself a fragment with
                    - various settings

        This structure can be used to easily draw paragraphs with the various alignments.
        You can supply either a single width or a list of widths; the latter will have its
        last item repeated until necessary. A 2-element list is useful when there is a
        different first line indent; a longer list could be created to facilitate custom wraps
        around irregular objects."""

        if not isSeq(width): maxWidths = [width]
        else: maxWidths = width
        lines = []
        lineno = 0
        maxWidth = maxWidths[lineno]
        style = self.style
        fFontSize = float(style.fontSize)
        requiredWidth = 0

        #for bullets, work out width and ensure we wrap the right amount onto line one
        _handleBulletWidth(self.bulletText,style,maxWidths)

        self.height = 0
        autoLeading = getattr(self,'autoLeading',getattr(style,'autoLeading',''))
        calcBounds = autoLeading not in ('','off')
        frags = self.frags
        nFrags= len(frags)
        if nFrags==1:
            f = frags[0]
            if hasattr(f,'text'):
                fontSize = f.fontSize
                fontName = f.fontName
                ascent, descent = getAscentDescent(fontName,fontSize)
                kind = 0
                L=f.text.split('\n')
                for l in L:
                    currentWidth = stringWidth(l,fontName,fontSize)
                    requiredWidth = max(currentWidth,requiredWidth)
                    extraSpace = maxWidth-currentWidth
                    lines.append((extraSpace,l.split(' '),currentWidth))
                    lineno = lineno+1
                    maxWidth = lineno<len(maxWidths) and maxWidths[lineno] or maxWidths[-1]
                blPara = f.clone(kind=kind, lines=lines,ascent=ascent,descent=descent,fontSize=fontSize)
            else:
                kind = f.kind
                lines = f.lines
                for L in lines:
                    if kind==0:
                        currentWidth = L[2]
                    else:
                        currentWidth = L.currentWidth
                    requiredWidth = max(currentWidth,requiredWidth)
                blPara = f.clone(kind=kind, lines=lines)

            self.width = max(self.width,requiredWidth)
            return blPara
        elif nFrags<=0:
            return ParaLines(kind=0, fontSize=style.fontSize, fontName=style.fontName,
                            textColor=style.textColor, ascent=style.fontSize,descent=-0.2*style.fontSize,
                            lines=[])
        else:
            for L in _getFragLines(frags):
                currentWidth, n, w = _getFragWord(L,maxWidth)
                f = w[0][0]
                maxSize = f.fontSize
                maxAscent, minDescent = getAscentDescent(f.fontName,maxSize)
                words = [f.clone()]
                words[-1].text = w[0][1]
                for i in w[1:]:
                    f = i[0].clone()
                    f.text=i[1]
                    words.append(f)
                    fontSize = f.fontSize
                    fontName = f.fontName
                    if calcBounds:
                        cbDefn = getattr(f,'cbDefn',None)
                        if getattr(cbDefn,'width',0):
                            descent,ascent = imgVRange(imgNormV(cbDefn.height,fontSize),cbDefn.valign,fontSize)
                        else:
                            ascent, descent = getAscentDescent(fontName,fontSize)
                    else:
                        ascent, descent = getAscentDescent(fontName,fontSize)
                    maxSize = max(maxSize,fontSize)
                    maxAscent = max(maxAscent,ascent)
                    minDescent = min(minDescent,descent)

                lineno += 1
                maxWidth = lineno<len(maxWidths) and maxWidths[lineno] or maxWidths[-1]
                requiredWidth = max(currentWidth,requiredWidth)
                extraSpace = maxWidth - currentWidth
                lines.append(ParaLines(extraSpace=extraSpace,wordCount=n, words=words, fontSize=maxSize, ascent=maxAscent,descent=minDescent,currentWidth=currentWidth,preformatted=True))

            self.width = max(self.width,requiredWidth)
            return ParaLines(kind=1, lines=lines)

        return lines

    breakLinesCJK = breakLines  #TODO fixme fixme fixme

    # we need this her to get the right splitter
    def _get_split_blParaFunc(self):
        return _split_blPara

class PythonPreformatted(XPreformatted):
    """Used for syntax-colored Python code, otherwise like XPreformatted.
    """
    formats = {
        'rest'       : ('', ''),
        'comment'    : ('<font color="green">', '</font>'),
        'keyword'    : ('<font color="blue"><b>', '</b></font>'),
        'parameter'  : ('<font color="black">', '</font>'),
        'identifier' : ('<font color="red">', '</font>'),
        'string'     : ('<font color="gray">', '</font>') }

    def __init__(self, text, style, bulletText = None, dedent=0, frags=None):
        if text:
            text = self.fontify(self.escapeHtml(text))
        XPreformatted.__init__(self, text, style,bulletText=bulletText,dedent=dedent,frags=frags)

    def escapeHtml(self, text):
        s = text.replace('&', '&amp;')
        s = s.replace('<', '&lt;')
        s = s.replace('>', '&gt;')
        return s

    def fontify(self, code):
        "Return a fontified version of some Python code."

        if code[0] == '\n':
            code = code[1:]

        tags = PyFontify.fontify(code)
        fontifiedCode = ''
        pos = 0
        for k, i, j, dummy in tags:
            fontifiedCode = fontifiedCode + code[pos:i]
            s, e = self.formats[k]
            fontifiedCode = fontifiedCode + s + code[i:j] + e
            pos = j

        fontifiedCode = fontifiedCode + code[pos:]

        return fontifiedCode

if __name__=='__main__':    #NORUNTESTS
    import sys
    def dumpXPreformattedLines(P):
        print('\n############dumpXPreforemattedLines(%s)' % str(P))
        lines = P.blPara.lines
        n =len(lines)
        outw=sys.stdout.write
        for l in range(n):
            line = lines[l]
            words = line.words
            nwords = len(words)
            outw('line%d: %d(%d)\n  ' % (l,nwords,line.wordCount))
            for w in range(nwords):
                outw(" %d:'%s'"%(w,words[w].text))
            print()

    def dumpXPreformattedFrags(P):
        print('\n############dumpXPreforemattedFrags(%s)' % str(P))
        frags = P.frags
        n =len(frags)
        for l in range(n):
            print("frag%d: '%s'" % (l, frags[l].text))

        outw=sys.stdout.write
        l = 0
        for L in _getFragLines(frags):
            n=0
            for W in _getFragWords(L,360):
                outw("frag%d.%d: size=%d" % (l, n, W[0]))
                n = n + 1
                for w in W[1:]:
                    outw(" '%s'" % w[1])
                print()
            l = l + 1

    def try_it(text,style,dedent,aW,aH):
        P=XPreformatted(text,style,dedent=dedent)
        dumpXPreformattedFrags(P)
        w,h = P.wrap(aW, aH)
        dumpXPreformattedLines(P)
        S = P.split(aW,aH)
        dumpXPreformattedLines(P)
        for s in S:
            s.wrap(aW,aH)
            dumpXPreformattedLines(s)
            aH = 500

    from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
    styleSheet = getSampleStyleSheet()
    B = styleSheet['BodyText']
    DTstyle = ParagraphStyle("discussiontext", parent=B)
    DTstyle.fontName= 'Helvetica'
    for (text,dedent,style, aW, aH, active) in [('''


The <font name=courier color=green>CMYK</font> or subtractive

method follows the way a printer
mixes three pigments (cyan, magenta, and yellow) to form colors.
Because mixing chemicals is more difficult than combining light there
is a fourth parameter for darkness.  For example a chemical
combination of the <font name=courier color=green>CMY</font> pigments generally never makes a perfect

black -- instead producing a muddy color -- so, to get black printers
don't use the <font name=courier color=green>CMY</font> pigments but use a direct black ink.  Because
<font name=courier color=green>CMYK</font> maps more directly to the way printer hardware works it may
be the case that &amp;| &amp; | colors specified in <font name=courier color=green>CMYK</font> will provide better fidelity
and better control when printed.


''',0,DTstyle, 456.0, 42.8, 0),
('''

   This is a non rearranging form of the <b>Paragraph</b> class;
   <b><font color=red>XML</font></b> tags are allowed in <i>text</i> and have the same

      meanings as for the <b>Paragraph</b> class.
   As for <b>Preformatted</b>, if dedent is non zero <font color=red size=+1>dedent</font>
       common leading spaces will be removed from the
   front of each line.

''',3, DTstyle, 456.0, 42.8, 0),
("""\
    <font color=blue>class </font><font color=red>FastXMLParser</font>:
        # Nonsense method
        def nonsense(self):
            self.foo = 'bar'
""",0, styleSheet['Code'], 456.0, 4.8, 1),
]:
        if active: try_it(text,style,dedent,aW,aH)



print("import: 'reportlab'")
import reportlab

print("import: 'reportlab.graphics.charts'")
import reportlab.graphics.charts

print("import: 'reportlab.graphics.samples'")
import reportlab.graphics.samples

print("import: 'reportlab.graphics.widgets'")
import reportlab.graphics.widgets

print("import: 'reportlab.graphics.barcode'")
import reportlab.graphics.barcode

print("import: 'reportlab.graphics'")
import reportlab.graphics

print("import: 'reportlab.lib'")
import reportlab.lib

print("import: 'reportlab.pdfbase'")
import reportlab.pdfbase

print("import: 'reportlab.pdfgen'")
import reportlab.pdfgen

print("import: 'reportlab.platypus'")
import reportlab.platypus




#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
#history https://bitbucket.org/rptlab/reportlab/history-node/tip/src/reportlab/pdfgen/pathobject.py
__version__='3.3.0'
__doc__="""
PDFPathObject is an efficient way to draw paths on a Canvas. Do not
instantiate directly, obtain one from the Canvas instead.

Progress Reports:
8.83, 2000-01-13, gmcm: created from pdfgen.py

"""

from reportlab.pdfgen import pdfgeom
from reportlab.lib.rl_accel import fp_str


class PDFPathObject:
    """Represents a graphic path.  There are certain 'modes' to PDF
    drawing, and making a separate object to expose Path operations
    ensures they are completed with no run-time overhead.  Ask
    the Canvas for a PDFPath with getNewPathObject(); moveto/lineto/
    curveto wherever you want; add whole shapes; and then add it back
    into the canvas with one of the relevant operators.

    Path objects are probably not long, so we pack onto one line

    the code argument allows a canvas to get the operatiosn appended directly so
    avoiding the final getCode
    """
    def __init__(self,code=None):
        self._code = (code,[])[code is None]
        self._code_append = self._init_code_append

    def _init_code_append(self,c):
        assert c.endswith(' m') or c.endswith(' re'), 'path must start with a moveto or rect'
        code_append = self._code.append
        code_append('n')
        code_append(c)
        self._code_append = code_append

    def getCode(self):
        "pack onto one line; used internally"
        return ' '.join(self._code)

    def moveTo(self, x, y):
        self._code_append('%s m' % fp_str(x,y))

    def lineTo(self, x, y):
        self._code_append('%s l' % fp_str(x,y))

    def curveTo(self, x1, y1, x2, y2, x3, y3):
        self._code_append('%s c' % fp_str(x1, y1, x2, y2, x3, y3))

    def arc(self, x1,y1, x2,y2, startAng=0, extent=90):
        """Contributed to piddlePDF by Robert Kern, 28/7/99.
        Draw a partial ellipse inscribed within the rectangle x1,y1,x2,y2,
        starting at startAng degrees and covering extent degrees.   Angles
        start with 0 to the right (+x) and increase counter-clockwise.
        These should have x1<x2 and y1<y2.

        The algorithm is an elliptical generalization of the formulae in
        Jim Fitzsimmon's TeX tutorial <URL: http://www.tinaja.com/bezarc1.pdf>."""

        self._curves(pdfgeom.bezierArc(x1,y1, x2,y2, startAng, extent))

    def arcTo(self, x1,y1, x2,y2, startAng=0, extent=90):
        """Like arc, but draws a line from the current point to
        the start if the start is not the current point."""
        self._curves(pdfgeom.bezierArc(x1,y1, x2,y2, startAng, extent),'lineTo')

    def rect(self, x, y, width, height):
        """Adds a rectangle to the path"""
        self._code_append('%s re' % fp_str((x, y, width, height)))

    def ellipse(self, x, y, width, height):
        """adds an ellipse to the path"""
        self._curves(pdfgeom.bezierArc(x, y, x + width,y + height, 0, 360))

    def _curves(self,curves,initial='moveTo'):
        getattr(self,initial)(*curves[0][:2])
        for curve in curves:
            self.curveTo(*curve[2:])

    def circle(self, x_cen, y_cen, r):
        """adds a circle to the path"""
        x1 = x_cen - r
        y1 = y_cen - r
        width = height = 2*r
        self.ellipse(x1, y1, width, height)

    def roundRect(self, x, y, width, height, radius):
        """Draws a rectangle with rounded corners. The corners are
        approximately quadrants of a circle, with the given radius."""
        #use a precomputed set of factors for the bezier approximation
        #to a circle. There are six relevant points on the x axis and y axis.
        #sketch them and it should all make sense!
        t = 0.4472 * radius

        x0 = x
        x1 = x0 + t
        x2 = x0 + radius
        x3 = x0 + width - radius
        x4 = x0 + width - t
        x5 = x0 + width

        y0 = y
        y1 = y0 + t
        y2 = y0 + radius
        y3 = y0 + height - radius
        y4 = y0 + height - t
        y5 = y0 + height

        self.moveTo(x2, y0)
        self.lineTo(x3, y0) #bottom row
        self.curveTo(x4, y0, x5, y1, x5, y2) #bottom right
        self.lineTo(x5, y3) #right edge
        self.curveTo(x5, y4, x4, y5, x3, y5) #top right
        self.lineTo(x2, y5) #top row
        self.curveTo(x1, y5, x0, y4, x0, y3) #top left
        self.lineTo(x0, y2) #left edge
        self.curveTo(x0, y1, x1, y0, x2, y0) #bottom left
        self.close()

    def close(self):
        "draws a line back to where it started"
        self._code_append('h')



MacRomanEncoding = (
                 None, None, None, None, None, None, None, None, None, None, None, None,
                 None, None, None, None, None, None, None, None, None, None, None, None,
                 None, None, None, None, None, None, None, None, 'space', 'exclam',
                 'quotedbl', 'numbersign', 'dollar', 'percent', 'ampersand',
                 'quotesingle', 'parenleft', 'parenright', 'asterisk', 'plus', 'comma',
                 'hyphen', 'period', 'slash', 'zero', 'one', 'two', 'three', 'four',
                 'five', 'six', 'seven', 'eight', 'nine', 'colon', 'semicolon', 'less',
                 'equal', 'greater', 'question', 'at', 'A', 'B', 'C', 'D', 'E', 'F',
                 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
                 'U', 'V', 'W', 'X', 'Y', 'Z', 'bracketleft', 'backslash', 'bracketright',
                 'asciicircum', 'underscore', 'grave', 'a', 'b', 'c', 'd', 'e', 'f',
                 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
                 'u', 'v', 'w', 'x', 'y', 'z', 'braceleft', 'bar', 'braceright',
                 'asciitilde', None, 'Adieresis', 'Aring', 'Ccedilla', 'Eacute',
                 'Ntilde', 'Odieresis', 'Udieresis', 'aacute', 'agrave', 'acircumflex',
                 'adieresis', 'atilde', 'aring', 'ccedilla', 'eacute', 'egrave',
                 'ecircumflex', 'edieresis', 'iacute', 'igrave', 'icircumflex',
                 'idieresis', 'ntilde', 'oacute', 'ograve', 'ocircumflex', 'odieresis',
                 'otilde', 'uacute', 'ugrave', 'ucircumflex', 'udieresis', 'dagger',
                 'degree', 'cent', 'sterling', 'section', 'bullet', 'paragraph',
                 'germandbls', 'registered', 'copyright', 'trademark', 'acute',
                 'dieresis', None, 'AE', 'Oslash', None, 'plusminus', None, None, 'yen',
                 'mu', None, None, None, None, None, 'ordfeminine', 'ordmasculine', None,
                 'ae', 'oslash', 'questiondown', 'exclamdown', 'logicalnot', None, 'florin',
                 None, None, 'guillemotleft', 'guillemotright', 'ellipsis', 'space', 'Agrave',
                 'Atilde', 'Otilde', 'OE', 'oe', 'endash', 'emdash', 'quotedblleft',
                 'quotedblright', 'quoteleft', 'quoteright', 'divide', None, 'ydieresis',
                 'Ydieresis', 'fraction', 'currency', 'guilsinglleft', 'guilsinglright',
                 'fi', 'fl', 'daggerdbl', 'periodcentered', 'quotesinglbase',
                 'quotedblbase', 'perthousand', 'Acircumflex', 'Ecircumflex', 'Aacute',
                 'Edieresis', 'Egrave', 'Iacute', 'Icircumflex', 'Idieresis', 'Igrave',
                 'Oacute', 'Ocircumflex', None, 'Ograve', 'Uacute', 'Ucircumflex',
                 'Ugrave', 'dotlessi', 'circumflex', 'tilde', 'macron', 'breve',
                 'dotaccent', 'ring', 'cedilla', 'hungarumlaut', 'ogonek', 'caron')



#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
# $URI:$
__version__='3.3.0'
__doc__='''Helper for the test suite - determines where to write output.

When our test suite runs as source, a script "test_foo.py" will typically
create "test_foo.pdf" alongside it.  But if you are testing a package of
compiled code inside a zip archive, this won't work.  This determines
where to write test suite output, creating a subdirectory of /tmp/ or
whatever if needed.

'''
_rl_tempdir=None
__all__ = ('get_rl_tempdir', 'get_rl_tempdir')
import os, tempfile
def _rl_getuid():
    if hasattr(os,'getuid'):
        return os.getuid()
    else:
        return ''

def get_rl_tempdir(*subdirs):
    global _rl_tempdir
    if _rl_tempdir is None:
        _rl_tempdir = os.path.join(tempfile.gettempdir(),'ReportLab_tmp%s' % str(_rl_getuid()))
    d = _rl_tempdir
    if subdirs: d = os.path.join(*((d,)+subdirs))
    try:
        os.makedirs(d)
    except:
        pass
    return d

def get_rl_tempfile(fn=None):
    if not fn:
        fn = tempfile.mktemp()
    return os.path.join(get_rl_tempdir(),fn)



#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
#history https://bitbucket.org/rptlab/reportlab/history-node/tip/src/reportlab/graphics/widgets/grids.py
__version__='3.3.0'

from reportlab.lib import colors
from reportlab.lib.validators import isNumber, isColorOrNone, isBoolean, isListOfNumbers, OneOf, isListOfColors, isNumberOrNone
from reportlab.lib.attrmap import AttrMap, AttrMapValue
from reportlab.graphics.shapes import Drawing, Group, Line, Rect, LineShape, definePath, EmptyClipPath
from reportlab.graphics.widgetbase import Widget

def frange(start, end=None, inc=None):
    "A range function, that does accept float increments..."

    if end == None:
        end = start + 0.0
        start = 0.0

    if inc == None:
        inc = 1.0

    L = []
    end = end - inc*0.0001  #to avoid numrical problems
    while 1:
        next = start + len(L) * inc
        if inc > 0 and next >= end:
            break
        elif inc < 0 and next <= end:
            break
        L.append(next)

    return L


def makeDistancesList(list):
    """Returns a list of distances between adjacent numbers in some input list.

    E.g. [1, 1, 2, 3, 5, 7] -> [0, 1, 1, 2, 2]
    """

    d = []
    for i in range(len(list[:-1])):
        d.append(list[i+1] - list[i])

    return d


class Grid(Widget):
    """This makes a rectangular grid of equidistant stripes.

    The grid contains an outer border rectangle, and stripes
    inside which can be drawn with lines and/or as solid tiles.
    The drawing order is: outer rectangle, then lines and tiles.

    The stripes' width is indicated as 'delta'. The sequence of
    stripes can have an offset named 'delta0'. Both values need
    to be positive!
    """

    _attrMap = AttrMap(
        x = AttrMapValue(isNumber, desc="The grid's lower-left x position."),
        y = AttrMapValue(isNumber, desc="The grid's lower-left y position."),
        width = AttrMapValue(isNumber, desc="The grid's width."),
        height = AttrMapValue(isNumber, desc="The grid's height."),
        orientation = AttrMapValue(OneOf(('vertical', 'horizontal')),
            desc='Determines if stripes are vertical or horizontal.'),
        useLines = AttrMapValue(OneOf((0, 1)),
            desc='Determines if stripes are drawn with lines.'),
        useRects = AttrMapValue(OneOf((0, 1)),
            desc='Determines if stripes are drawn with solid rectangles.'),
        delta = AttrMapValue(isNumber,
            desc='Determines the width/height of the stripes.'),
        delta0 = AttrMapValue(isNumber,
            desc='Determines the stripes initial width/height offset.'),
        deltaSteps = AttrMapValue(isListOfNumbers,
            desc='List of deltas to be used cyclically.'),
        stripeColors = AttrMapValue(isListOfColors,
            desc='Colors applied cyclically in the right or upper direction.'),
        fillColor = AttrMapValue(isColorOrNone,
            desc='Background color for entire rectangle.'),
        strokeColor = AttrMapValue(isColorOrNone,
            desc='Color used for lines.'),
        strokeWidth = AttrMapValue(isNumber,
            desc='Width used for lines.'),
        rectStrokeColor = AttrMapValue(isColorOrNone, desc='Color for outer rect stroke.'),
        rectStrokeWidth = AttrMapValue(isNumberOrNone, desc='Width for outer rect stroke.'),
        )

    def __init__(self):
        self.x = 0
        self.y = 0
        self.width = 100
        self.height = 100
        self.orientation = 'vertical'
        self.useLines = 0
        self.useRects = 1
        self.delta = 20
        self.delta0 = 0
        self.deltaSteps = []
        self.fillColor = colors.white
        self.stripeColors = [colors.red, colors.green, colors.blue]
        self.strokeColor = colors.black
        self.strokeWidth = 2


    def demo(self):
        D = Drawing(100, 100)

        g = Grid()
        D.add(g)

        return D

    def makeOuterRect(self):
        strokeColor = getattr(self,'rectStrokeColor',self.strokeColor)
        strokeWidth = getattr(self,'rectStrokeWidth',self.strokeWidth)
        if self.fillColor or (strokeColor and strokeWidth):
            rect = Rect(self.x, self.y, self.width, self.height)
            rect.fillColor = self.fillColor
            rect.strokeColor = strokeColor
            rect.strokeWidth = strokeWidth
            return rect
        else:
            return None

    def makeLinePosList(self, start, isX=0):
        "Returns a list of positions where to place lines."

        w, h = self.width, self.height
        if isX:
            length = w
        else:
            length = h
        if self.deltaSteps:
            r = [start + self.delta0]
            i = 0
            while 1:
                if r[-1] > start + length:
                    del r[-1]
                    break
                r.append(r[-1] + self.deltaSteps[i % len(self.deltaSteps)])
                i = i + 1
        else:
            r = frange(start + self.delta0, start + length, self.delta)

        r.append(start + length)
        if self.delta0 != 0:
            r.insert(0, start)
        #print 'Grid.makeLinePosList() -> %s' % r
        return r


    def makeInnerLines(self):
        # inner grid lines
        group = Group()

        w, h = self.width, self.height

        if self.useLines == 1:
            if self.orientation == 'vertical':
                r = self.makeLinePosList(self.x, isX=1)
                for x in r:
                    line = Line(x, self.y, x, self.y + h)
                    line.strokeColor = self.strokeColor
                    line.strokeWidth = self.strokeWidth
                    group.add(line)
            elif self.orientation == 'horizontal':
                r = self.makeLinePosList(self.y, isX=0)
                for y in r:
                    line = Line(self.x, y, self.x + w, y)
                    line.strokeColor = self.strokeColor
                    line.strokeWidth = self.strokeWidth
                    group.add(line)

        return group


    def makeInnerTiles(self):
        # inner grid lines
        group = Group()

        w, h = self.width, self.height

        # inner grid stripes (solid rectangles)
        if self.useRects == 1:
            cols = self.stripeColors

            if self.orientation == 'vertical':
                r = self.makeLinePosList(self.x, isX=1)
            elif self.orientation == 'horizontal':
                r = self.makeLinePosList(self.y, isX=0)

            dist = makeDistancesList(r)

            i = 0
            for j in range(len(dist)):
                if self.orientation == 'vertical':
                    x = r[j]
                    stripe = Rect(x, self.y, dist[j], h)
                elif self.orientation == 'horizontal':
                    y = r[j]
                    stripe = Rect(self.x, y, w, dist[j])
                stripe.fillColor = cols[i % len(cols)]
                stripe.strokeColor = None
                group.add(stripe)
                i = i + 1

        return group


    def draw(self):
        # general widget bits
        group = Group()

        group.add(self.makeOuterRect())
        group.add(self.makeInnerTiles())
        group.add(self.makeInnerLines(),name='_gridLines')

        return group


class DoubleGrid(Widget):
    """This combines two ordinary Grid objects orthogonal to each other.
    """

    _attrMap = AttrMap(
        x = AttrMapValue(isNumber, desc="The grid's lower-left x position."),
        y = AttrMapValue(isNumber, desc="The grid's lower-left y position."),
        width = AttrMapValue(isNumber, desc="The grid's width."),
        height = AttrMapValue(isNumber, desc="The grid's height."),
        grid0 = AttrMapValue(None, desc="The first grid component."),
        grid1 = AttrMapValue(None, desc="The second grid component."),
        )

    def __init__(self):
        self.x = 0
        self.y = 0
        self.width = 100
        self.height = 100

        g0 = Grid()
        g0.x = self.x
        g0.y = self.y
        g0.width = self.width
        g0.height = self.height
        g0.orientation = 'vertical'
        g0.useLines = 1
        g0.useRects = 0
        g0.delta = 20
        g0.delta0 = 0
        g0.deltaSteps = []
        g0.fillColor = colors.white
        g0.stripeColors = [colors.red, colors.green, colors.blue]
        g0.strokeColor = colors.black
        g0.strokeWidth = 1

        g1 = Grid()
        g1.x = self.x
        g1.y = self.y
        g1.width = self.width
        g1.height = self.height
        g1.orientation = 'horizontal'
        g1.useLines = 1
        g1.useRects = 0
        g1.delta = 20
        g1.delta0 = 0
        g1.deltaSteps = []
        g1.fillColor = colors.white
        g1.stripeColors = [colors.red, colors.green, colors.blue]
        g1.strokeColor = colors.black
        g1.strokeWidth = 1

        self.grid0 = g0
        self.grid1 = g1


##    # This gives an AttributeError:
##    #   DoubleGrid instance has no attribute 'grid0'
##    def __setattr__(self, name, value):
##        if name in ('x', 'y', 'width', 'height'):
##            setattr(self.grid0, name, value)
##            setattr(self.grid1, name, value)


    def demo(self):
        D = Drawing(100, 100)
        g = DoubleGrid()
        D.add(g)
        return D


    def draw(self):
        group = Group()
        g0, g1 = self.grid0, self.grid1
        # Order groups to make sure both v and h lines
        # are visible (works only when there is only
        # one kind of stripes, v or h).
        G = g0.useRects == 1 and g1.useRects == 0 and (g0,g1) or (g1,g0)
        for g in G:
            group.add(g.makeOuterRect())
        for g in G:
            group.add(g.makeInnerTiles())
            group.add(g.makeInnerLines(),name='_gridLines')

        return group


class ShadedRect(Widget):
    """This makes a rectangle with shaded colors between two colors.

    Colors are interpolated linearly between 'fillColorStart'
    and 'fillColorEnd', both of which appear at the margins.
    If 'numShades' is set to one, though, only 'fillColorStart'
    is used.
    """

    _attrMap = AttrMap(
        x = AttrMapValue(isNumber, desc="The grid's lower-left x position."),
        y = AttrMapValue(isNumber, desc="The grid's lower-left y position."),
        width = AttrMapValue(isNumber, desc="The grid's width."),
        height = AttrMapValue(isNumber, desc="The grid's height."),
        orientation = AttrMapValue(OneOf(('vertical', 'horizontal')), desc='Determines if stripes are vertical or horizontal.'),
        numShades = AttrMapValue(isNumber, desc='The number of interpolating colors.'),
        fillColorStart = AttrMapValue(isColorOrNone, desc='Start value of the color shade.'),
        fillColorEnd = AttrMapValue(isColorOrNone, desc='End value of the color shade.'),
        strokeColor = AttrMapValue(isColorOrNone, desc='Color used for border line.'),
        strokeWidth = AttrMapValue(isNumber, desc='Width used for lines.'),
        cylinderMode = AttrMapValue(isBoolean, desc='True if shading reverses in middle.'),
        )

    def __init__(self,**kw):
        self.x = 0
        self.y = 0
        self.width = 100
        self.height = 100
        self.orientation = 'vertical'
        self.numShades = 20
        self.fillColorStart = colors.pink
        self.fillColorEnd = colors.black
        self.strokeColor = colors.black
        self.strokeWidth = 2
        self.cylinderMode = 0
        self.setProperties(kw)

    def demo(self):
        D = Drawing(100, 100)
        g = ShadedRect()
        D.add(g)

        return D

    def _flipRectCorners(self):
        "Flip rectangle's corners if width or height is negative."
        x, y, width, height, fillColorStart, fillColorEnd = self.x, self.y, self.width, self.height, self.fillColorStart, self.fillColorEnd
        if width < 0 and height > 0:
            x = x + width
            width = -width
            if self.orientation=='vertical': fillColorStart, fillColorEnd = fillColorEnd, fillColorStart
        elif height<0 and width>0:
            y = y + height
            height = -height
            if self.orientation=='horizontal': fillColorStart, fillColorEnd = fillColorEnd, fillColorStart
        elif height < 0 and height < 0:
            x = x + width
            width = -width
            y = y + height
            height = -height
        return x, y, width, height, fillColorStart, fillColorEnd

    def draw(self):
        # general widget bits
        group = Group()
        x, y, w, h, c0, c1 = self._flipRectCorners()
        numShades = self.numShades
        if self.cylinderMode:
            if not numShades%2: numShades = numShades+1
            halfNumShades = int((numShades-1)/2) + 1
        num = float(numShades) # must make it float!
        vertical = self.orientation == 'vertical'
        if vertical:
            if numShades == 1:
                V = [x]
            else:
                V = frange(x, x + w, w/num)
        else:
            if numShades == 1:
                V = [y]
            else:
                V = frange(y, y + h, h/num)

        for v in V:
            stripe = vertical and Rect(v, y, w/num, h) or Rect(x, v, w, h/num)
            if self.cylinderMode:
                if V.index(v)>=halfNumShades:
                    col = colors.linearlyInterpolatedColor(c1,c0,V[halfNumShades],V[-1], v)
                else:
                    col = colors.linearlyInterpolatedColor(c0,c1,V[0],V[halfNumShades], v)
            else:
                col = colors.linearlyInterpolatedColor(c0,c1,V[0],V[-1], v)
            stripe.fillColor = col
            stripe.strokeColor = col
            stripe.strokeWidth = 1
            group.add(stripe)
        if self.strokeColor and self.strokeWidth>=0:
            rect = Rect(x, y, w, h)
            rect.strokeColor = self.strokeColor
            rect.strokeWidth = self.strokeWidth
            rect.fillColor = None
            group.add(rect)
        return group


def colorRange(c0, c1, n):
    "Return a range of intermediate colors between c0 and c1"
    if n==1: return [c0]

    C = []
    if n>1:
        lim = n-1
        for i in range(n):
            C.append(colors.linearlyInterpolatedColor(c0,c1,0,lim, i))
    return C


def centroid(P):
    '''compute average point of a set of points'''
    cx = 0
    cy = 0
    for x,y in P:
        cx+=x
        cy+=y
    n = float(len(P))
    return cx/n, cy/n

def rotatedEnclosingRect(P, angle, rect):
    '''
    given P a sequence P of x,y coordinate pairs and an angle in degrees
    find the centroid of P and the axis at angle theta through it
    find the extreme points of P wrt axis parallel distance and axis
    orthogonal distance. Then compute the least rectangle that will still
    enclose P when rotated by angle.

    The class R
    '''
    from math import pi, cos, sin, tan
    x0, y0 = centroid(P)
    theta = (angle/180.)*pi
    s,c=sin(theta),cos(theta)
    def parallelAxisDist(xy,s=s,c=c,x0=x0,y0=y0):
        x,y = xy
        return (s*(y-y0)+c*(x-x0))
    def orthogonalAxisDist(xy,s=s,c=c,x0=x0,y0=y0):
        x,y = xy
        return (c*(y-y0)+s*(x-x0))
    L = list(map(parallelAxisDist,P))
    L.sort()
    a0, a1 = L[0], L[-1]
    L = list(map(orthogonalAxisDist,P))
    L.sort()
    b0, b1 = L[0], L[-1]
    rect.x, rect.width = a0, a1-a0
    rect.y, rect.height = b0, b1-b0
    g = Group(transform=(c,s,-s,c,x0,y0))
    g.add(rect)
    return g

class ShadedPolygon(Widget,LineShape):
    _attrMap = AttrMap(BASE=LineShape,
        angle = AttrMapValue(isNumber,desc="Shading angle"),
        fillColorStart = AttrMapValue(isColorOrNone),
        fillColorEnd = AttrMapValue(isColorOrNone),
        numShades = AttrMapValue(isNumber, desc='The number of interpolating colors.'),
        cylinderMode = AttrMapValue(isBoolean, desc='True if shading reverses in middle.'),
        points = AttrMapValue(isListOfNumbers),
        )

    def __init__(self,**kw):
        self.angle = 90
        self.fillColorStart = colors.red
        self.fillColorEnd = colors.green
        self.cylinderMode = 0
        self.numShades = 50
        self.points = [-1,-1,2,2,3,-1]
        LineShape.__init__(self,kw)

    def draw(self):
        P = self.points
        P = list(map(lambda i, P=P:(P[i],P[i+1]),range(0,len(P),2)))
        path = definePath([('moveTo',)+P[0]]+[('lineTo',)+x for x in P[1:]]+['closePath'],
            fillColor=None, strokeColor=None)
        path.isClipPath = 1
        g = Group()
        g.add(path)
        angle = self.angle
        orientation = 'vertical'
        if angle==180:
            angle = 0
        elif angle in (90,270):
            orientation ='horizontal'
            angle = 0
        rect = ShadedRect(strokeWidth=0,strokeColor=None,orientation=orientation)
        for k in 'fillColorStart', 'fillColorEnd', 'numShades', 'cylinderMode':
            setattr(rect,k,getattr(self,k))
        g.add(rotatedEnclosingRect(P, angle, rect))
        g.add(EmptyClipPath)
        path = path.copy()
        path.isClipPath = 0
        path.strokeColor = self.strokeColor
        path.strokeWidth = self.strokeWidth
        g.add(path)
        return g

if __name__=='__main__': #noruntests
    from reportlab.lib.colors import blue
    from reportlab.graphics.shapes import Drawing
    angle=45
    D = Drawing(120,120)
    D.add(ShadedPolygon(points=(10,10,60,60,110,10),strokeColor=None,strokeWidth=1,angle=90,numShades=50,cylinderMode=0))
    D.save(formats=['gif'],fnRoot='shobj',outDir='/tmp')



#codecs support
__all__=['RL_Codecs']
from collections import namedtuple
import codecs
StdCodecData=namedtuple('StdCodecData','exceptions rexceptions')
ExtCodecData=namedtuple('ExtCodecData','baseName exceptions rexceptions')
class RL_Codecs:
    __rl_codecs_data = {
        'winansi':StdCodecData({
            0x007f: 0x2022, # BULLET
            0x0080: 0x20ac, # EURO SIGN
            0x0081: 0x2022, # BULLET
            0x0082: 0x201a, # SINGLE LOW-9 QUOTATION MARK
            0x0083: 0x0192, # LATIN SMALL LETTER F WITH HOOK
            0x0084: 0x201e, # DOUBLE LOW-9 QUOTATION MARK
            0x0085: 0x2026, # HORIZONTAL ELLIPSIS
            0x0086: 0x2020, # DAGGER
            0x0087: 0x2021, # DOUBLE DAGGER
            0x0088: 0x02c6, # MODIFIER LETTER CIRCUMFLEX ACCENT
            0x0089: 0x2030, # PER MILLE SIGN
            0x008a: 0x0160, # LATIN CAPITAL LETTER S WITH CARON
            0x008b: 0x2039, # SINGLE LEFT-POINTING ANGLE QUOTATION MARK
            0x008c: 0x0152, # LATIN CAPITAL LIGATURE OE
            0x008d: 0x2022, # BULLET
            0x008e: 0x017d, # LATIN CAPITAL LETTER Z WITH CARON
            0x008f: 0x2022, # BULLET
            0x0090: 0x2022, # BULLET
            0x0091: 0x2018, # LEFT SINGLE QUOTATION MARK
            0x0092: 0x2019, # RIGHT SINGLE QUOTATION MARK
            0x0093: 0x201c, # LEFT DOUBLE QUOTATION MARK
            0x0094: 0x201d, # RIGHT DOUBLE QUOTATION MARK
            0x0095: 0x2022, # BULLET
            0x0096: 0x2013, # EN DASH
            0x0097: 0x2014, # EM DASH
            0x0098: 0x02dc, # SMALL TILDE
            0x0099: 0x2122, # TRADE MARK SIGN
            0x009a: 0x0161, # LATIN SMALL LETTER S WITH CARON
            0x009b: 0x203a, # SINGLE RIGHT-POINTING ANGLE QUOTATION MARK
            0x009c: 0x0153, # LATIN SMALL LIGATURE OE
            0x009d: 0x2022, # BULLET
            0x009e: 0x017e, # LATIN SMALL LETTER Z WITH CARON
            0x009f: 0x0178, # LATIN CAPITAL LETTER Y WITH DIAERESIS
            0x00a0: 0x0020, # SPACE
            }, {0x2022:0x7f,0x20:0x20,0xa0:0x20}),
        'macroman':StdCodecData({
            0x007f: None, # UNDEFINED
            0x0080: 0x00c4, # LATIN CAPITAL LETTER A WITH DIAERESIS
            0x0081: 0x00c5, # LATIN CAPITAL LETTER A WITH RING ABOVE
            0x0082: 0x00c7, # LATIN CAPITAL LETTER C WITH CEDILLA
            0x0083: 0x00c9, # LATIN CAPITAL LETTER E WITH ACUTE
            0x0084: 0x00d1, # LATIN CAPITAL LETTER N WITH TILDE
            0x0085: 0x00d6, # LATIN CAPITAL LETTER O WITH DIAERESIS
            0x0086: 0x00dc, # LATIN CAPITAL LETTER U WITH DIAERESIS
            0x0087: 0x00e1, # LATIN SMALL LETTER A WITH ACUTE
            0x0088: 0x00e0, # LATIN SMALL LETTER A WITH GRAVE
            0x0089: 0x00e2, # LATIN SMALL LETTER A WITH CIRCUMFLEX
            0x008a: 0x00e4, # LATIN SMALL LETTER A WITH DIAERESIS
            0x008b: 0x00e3, # LATIN SMALL LETTER A WITH TILDE
            0x008c: 0x00e5, # LATIN SMALL LETTER A WITH RING ABOVE
            0x008d: 0x00e7, # LATIN SMALL LETTER C WITH CEDILLA
            0x008e: 0x00e9, # LATIN SMALL LETTER E WITH ACUTE
            0x008f: 0x00e8, # LATIN SMALL LETTER E WITH GRAVE
            0x0090: 0x00ea, # LATIN SMALL LETTER E WITH CIRCUMFLEX
            0x0091: 0x00eb, # LATIN SMALL LETTER E WITH DIAERESIS
            0x0092: 0x00ed, # LATIN SMALL LETTER I WITH ACUTE
            0x0093: 0x00ec, # LATIN SMALL LETTER I WITH GRAVE
            0x0094: 0x00ee, # LATIN SMALL LETTER I WITH CIRCUMFLEX
            0x0095: 0x00ef, # LATIN SMALL LETTER I WITH DIAERESIS
            0x0096: 0x00f1, # LATIN SMALL LETTER N WITH TILDE
            0x0097: 0x00f3, # LATIN SMALL LETTER O WITH ACUTE
            0x0098: 0x00f2, # LATIN SMALL LETTER O WITH GRAVE
            0x0099: 0x00f4, # LATIN SMALL LETTER O WITH CIRCUMFLEX
            0x009a: 0x00f6, # LATIN SMALL LETTER O WITH DIAERESIS
            0x009b: 0x00f5, # LATIN SMALL LETTER O WITH TILDE
            0x009c: 0x00fa, # LATIN SMALL LETTER U WITH ACUTE
            0x009d: 0x00f9, # LATIN SMALL LETTER U WITH GRAVE
            0x009e: 0x00fb, # LATIN SMALL LETTER U WITH CIRCUMFLEX
            0x009f: 0x00fc, # LATIN SMALL LETTER U WITH DIAERESIS
            0x00a0: 0x2020, # DAGGER
            0x00a1: 0x00b0, # DEGREE SIGN
            0x00a4: 0x00a7, # SECTION SIGN
            0x00a5: 0x2022, # BULLET
            0x00a6: 0x00b6, # PILCROW SIGN
            0x00a7: 0x00df, # LATIN SMALL LETTER SHARP S
            0x00a8: 0x00ae, # REGISTERED SIGN
            0x00aa: 0x2122, # TRADE MARK SIGN
            0x00ab: 0x00b4, # ACUTE ACCENT
            0x00ac: 0x00a8, # DIAERESIS
            0x00ad: None, # UNDEFINED
            0x00ae: 0x00c6, # LATIN CAPITAL LETTER AE
            0x00af: 0x00d8, # LATIN CAPITAL LETTER O WITH STROKE
            0x00b0: None, # UNDEFINED
            0x00b2: None, # UNDEFINED
            0x00b3: None, # UNDEFINED
            0x00b4: 0x00a5, # YEN SIGN
            0x00b6: None, # UNDEFINED
            0x00b7: None, # UNDEFINED
            0x00b8: None, # UNDEFINED
            0x00b9: None, # UNDEFINED
            0x00ba: None, # UNDEFINED
            0x00bb: 0x00aa, # FEMININE ORDINAL INDICATOR
            0x00bc: 0x00ba, # MASCULINE ORDINAL INDICATOR
            0x00bd: None, # UNDEFINED
            0x00be: 0x00e6, # LATIN SMALL LETTER AE
            0x00bf: 0x00f8, # LATIN SMALL LETTER O WITH STROKE
            0x00c0: 0x00bf, # INVERTED QUESTION MARK
            0x00c1: 0x00a1, # INVERTED EXCLAMATION MARK
            0x00c2: 0x00ac, # NOT SIGN
            0x00c3: None, # UNDEFINED
            0x00c4: 0x0192, # LATIN SMALL LETTER F WITH HOOK
            0x00c5: None, # UNDEFINED
            0x00c6: None, # UNDEFINED
            0x00c7: 0x00ab, # LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
            0x00c8: 0x00bb, # RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
            0x00c9: 0x2026, # HORIZONTAL ELLIPSIS
            0x00ca: 0x0020, # SPACE
            0x00cb: 0x00c0, # LATIN CAPITAL LETTER A WITH GRAVE
            0x00cc: 0x00c3, # LATIN CAPITAL LETTER A WITH TILDE
            0x00cd: 0x00d5, # LATIN CAPITAL LETTER O WITH TILDE
            0x00ce: 0x0152, # LATIN CAPITAL LIGATURE OE
            0x00cf: 0x0153, # LATIN SMALL LIGATURE OE
            0x00d0: 0x2013, # EN DASH
            0x00d1: 0x2014, # EM DASH
            0x00d2: 0x201c, # LEFT DOUBLE QUOTATION MARK
            0x00d3: 0x201d, # RIGHT DOUBLE QUOTATION MARK
            0x00d4: 0x2018, # LEFT SINGLE QUOTATION MARK
            0x00d5: 0x2019, # RIGHT SINGLE QUOTATION MARK
            0x00d6: 0x00f7, # DIVISION SIGN
            0x00d7: None, # UNDEFINED
            0x00d8: 0x00ff, # LATIN SMALL LETTER Y WITH DIAERESIS
            0x00d9: 0x0178, # LATIN CAPITAL LETTER Y WITH DIAERESIS
            0x00da: 0x2044, # FRACTION SLASH
            0x00db: 0x00a4, # CURRENCY SIGN
            0x00dc: 0x2039, # SINGLE LEFT-POINTING ANGLE QUOTATION MARK
            0x00dd: 0x203a, # SINGLE RIGHT-POINTING ANGLE QUOTATION MARK
            0x00de: 0xfb01, # LATIN SMALL LIGATURE FI
            0x00df: 0xfb02, # LATIN SMALL LIGATURE FL
            0x00e0: 0x2021, # DOUBLE DAGGER
            0x00e1: 0x00b7, # MIDDLE DOT
            0x00e2: 0x201a, # SINGLE LOW-9 QUOTATION MARK
            0x00e3: 0x201e, # DOUBLE LOW-9 QUOTATION MARK
            0x00e4: 0x2030, # PER MILLE SIGN
            0x00e5: 0x00c2, # LATIN CAPITAL LETTER A WITH CIRCUMFLEX
            0x00e6: 0x00ca, # LATIN CAPITAL LETTER E WITH CIRCUMFLEX
            0x00e7: 0x00c1, # LATIN CAPITAL LETTER A WITH ACUTE
            0x00e8: 0x00cb, # LATIN CAPITAL LETTER E WITH DIAERESIS
            0x00e9: 0x00c8, # LATIN CAPITAL LETTER E WITH GRAVE
            0x00ea: 0x00cd, # LATIN CAPITAL LETTER I WITH ACUTE
            0x00eb: 0x00ce, # LATIN CAPITAL LETTER I WITH CIRCUMFLEX
            0x00ec: 0x00cf, # LATIN CAPITAL LETTER I WITH DIAERESIS
            0x00ed: 0x00cc, # LATIN CAPITAL LETTER I WITH GRAVE
            0x00ee: 0x00d3, # LATIN CAPITAL LETTER O WITH ACUTE
            0x00ef: 0x00d4, # LATIN CAPITAL LETTER O WITH CIRCUMFLEX
            0x00f0: None, # UNDEFINED
            0x00f1: 0x00d2, # LATIN CAPITAL LETTER O WITH GRAVE
            0x00f2: 0x00da, # LATIN CAPITAL LETTER U WITH ACUTE
            0x00f3: 0x00db, # LATIN CAPITAL LETTER U WITH CIRCUMFLEX
            0x00f4: 0x00d9, # LATIN CAPITAL LETTER U WITH GRAVE
            0x00f5: 0x0131, # LATIN SMALL LETTER DOTLESS I
            0x00f6: 0x02c6, # MODIFIER LETTER CIRCUMFLEX ACCENT
            0x00f7: 0x02dc, # SMALL TILDE
            0x00f8: 0x00af, # MACRON
            0x00f9: 0x02d8, # BREVE
            0x00fa: 0x02d9, # DOT ABOVE
            0x00fb: 0x02da, # RING ABOVE
            0x00fc: 0x00b8, # CEDILLA
            0x00fd: 0x02dd, # DOUBLE ACUTE ACCENT
            0x00fe: 0x02db, # OGONEK
            0x00ff: 0x02c7, # CARON
            },None),
    'standard':StdCodecData({
            0x0027: 0x2019, # RIGHT SINGLE QUOTATION MARK
            0x0060: 0x2018, # LEFT SINGLE QUOTATION MARK
            0x007f: None, # UNDEFINED
            0x0080: None, # UNDEFINED
            0x0081: None, # UNDEFINED
            0x0082: None, # UNDEFINED
            0x0083: None, # UNDEFINED
            0x0084: None, # UNDEFINED
            0x0085: None, # UNDEFINED
            0x0086: None, # UNDEFINED
            0x0087: None, # UNDEFINED
            0x0088: None, # UNDEFINED
            0x0089: None, # UNDEFINED
            0x008a: None, # UNDEFINED
            0x008b: None, # UNDEFINED
            0x008c: None, # UNDEFINED
            0x008d: None, # UNDEFINED
            0x008e: None, # UNDEFINED
            0x008f: None, # UNDEFINED
            0x0090: None, # UNDEFINED
            0x0091: None, # UNDEFINED
            0x0092: None, # UNDEFINED
            0x0093: None, # UNDEFINED
            0x0094: None, # UNDEFINED
            0x0095: None, # UNDEFINED
            0x0096: None, # UNDEFINED
            0x0097: None, # UNDEFINED
            0x0098: None, # UNDEFINED
            0x0099: None, # UNDEFINED
            0x009a: None, # UNDEFINED
            0x009b: None, # UNDEFINED
            0x009c: None, # UNDEFINED
            0x009d: None, # UNDEFINED
            0x009e: None, # UNDEFINED
            0x009f: None, # UNDEFINED
            0x00a0: None, # UNDEFINED
            0x00a4: 0x2044, # FRACTION SLASH
            0x00a6: 0x0192, # LATIN SMALL LETTER F WITH HOOK
            0x00a8: 0x00a4, # CURRENCY SIGN
            0x00a9: 0x0027, # APOSTROPHE
            0x00aa: 0x201c, # LEFT DOUBLE QUOTATION MARK
            0x00ac: 0x2039, # SINGLE LEFT-POINTING ANGLE QUOTATION MARK
            0x00ad: 0x203a, # SINGLE RIGHT-POINTING ANGLE QUOTATION MARK
            0x00ae: 0xfb01, # LATIN SMALL LIGATURE FI
            0x00af: 0xfb02, # LATIN SMALL LIGATURE FL
            0x00b0: None, # UNDEFINED
            0x00b1: 0x2013, # EN DASH
            0x00b2: 0x2020, # DAGGER
            0x00b3: 0x2021, # DOUBLE DAGGER
            0x00b4: 0x00b7, # MIDDLE DOT
            0x00b5: None, # UNDEFINED
            0x00b7: 0x2022, # BULLET
            0x00b8: 0x201a, # SINGLE LOW-9 QUOTATION MARK
            0x00b9: 0x201e, # DOUBLE LOW-9 QUOTATION MARK
            0x00ba: 0x201d, # RIGHT DOUBLE QUOTATION MARK
            0x00bc: 0x2026, # HORIZONTAL ELLIPSIS
            0x00bd: 0x2030, # PER MILLE SIGN
            0x00be: None, # UNDEFINED
            0x00c0: None, # UNDEFINED
            0x00c1: 0x0060, # GRAVE ACCENT
            0x00c2: 0x00b4, # ACUTE ACCENT
            0x00c3: 0x02c6, # MODIFIER LETTER CIRCUMFLEX ACCENT
            0x00c4: 0x02dc, # SMALL TILDE
            0x00c5: 0x00af, # MACRON
            0x00c6: 0x02d8, # BREVE
            0x00c7: 0x02d9, # DOT ABOVE
            0x00c8: 0x00a8, # DIAERESIS
            0x00c9: None, # UNDEFINED
            0x00ca: 0x02da, # RING ABOVE
            0x00cb: 0x00b8, # CEDILLA
            0x00cc: None, # UNDEFINED
            0x00cd: 0x02dd, # DOUBLE ACUTE ACCENT
            0x00ce: 0x02db, # OGONEK
            0x00cf: 0x02c7, # CARON
            0x00d0: 0x2014, # EM DASH
            0x00d1: None, # UNDEFINED
            0x00d2: None, # UNDEFINED
            0x00d3: None, # UNDEFINED
            0x00d4: None, # UNDEFINED
            0x00d5: None, # UNDEFINED
            0x00d6: None, # UNDEFINED
            0x00d7: None, # UNDEFINED
            0x00d8: None, # UNDEFINED
            0x00d9: None, # UNDEFINED
            0x00da: None, # UNDEFINED
            0x00db: None, # UNDEFINED
            0x00dc: None, # UNDEFINED
            0x00dd: None, # UNDEFINED
            0x00de: None, # UNDEFINED
            0x00df: None, # UNDEFINED
            0x00e0: None, # UNDEFINED
            0x00e1: 0x00c6, # LATIN CAPITAL LETTER AE
            0x00e2: None, # UNDEFINED
            0x00e3: 0x00aa, # FEMININE ORDINAL INDICATOR
            0x00e4: None, # UNDEFINED
            0x00e5: None, # UNDEFINED
            0x00e6: None, # UNDEFINED
            0x00e7: None, # UNDEFINED
            0x00e8: 0x0141, # LATIN CAPITAL LETTER L WITH STROKE
            0x00e9: 0x00d8, # LATIN CAPITAL LETTER O WITH STROKE
            0x00ea: 0x0152, # LATIN CAPITAL LIGATURE OE
            0x00eb: 0x00ba, # MASCULINE ORDINAL INDICATOR
            0x00ec: None, # UNDEFINED
            0x00ed: None, # UNDEFINED
            0x00ee: None, # UNDEFINED
            0x00ef: None, # UNDEFINED
            0x00f0: None, # UNDEFINED
            0x00f1: 0x00e6, # LATIN SMALL LETTER AE
            0x00f2: None, # UNDEFINED
            0x00f3: None, # UNDEFINED
            0x00f4: None, # UNDEFINED
            0x00f5: 0x0131, # LATIN SMALL LETTER DOTLESS I
            0x00f6: None, # UNDEFINED
            0x00f7: None, # UNDEFINED
            0x00f8: 0x0142, # LATIN SMALL LETTER L WITH STROKE
            0x00f9: 0x00f8, # LATIN SMALL LETTER O WITH STROKE
            0x00fa: 0x0153, # LATIN SMALL LIGATURE OE
            0x00fb: 0x00df, # LATIN SMALL LETTER SHARP S
            0x00fc: None, # UNDEFINED
            0x00fd: None, # UNDEFINED
            0x00fe: None, # UNDEFINED
            0x00ff: None, # UNDEFINED
            },None),
    'symbol':StdCodecData({
            0x0022: 0x2200, # FOR ALL
            0x0024: 0x2203, # THERE EXISTS
            0x0027: 0x220b, # CONTAINS AS MEMBER
            0x002a: 0x2217, # ASTERISK OPERATOR
            0x002d: 0x2212, # MINUS SIGN
            0x0040: 0x2245, # APPROXIMATELY EQUAL TO
            0x0041: 0x0391, # GREEK CAPITAL LETTER ALPHA
            0x0042: 0x0392, # GREEK CAPITAL LETTER BETA
            0x0043: 0x03a7, # GREEK CAPITAL LETTER CHI
            0x0044: 0x2206, # INCREMENT
            0x0045: 0x0395, # GREEK CAPITAL LETTER EPSILON
            0x0046: 0x03a6, # GREEK CAPITAL LETTER PHI
            0x0047: 0x0393, # GREEK CAPITAL LETTER GAMMA
            0x0048: 0x0397, # GREEK CAPITAL LETTER ETA
            0x0049: 0x0399, # GREEK CAPITAL LETTER IOTA
            0x004a: 0x03d1, # GREEK THETA SYMBOL
            0x004b: 0x039a, # GREEK CAPITAL LETTER KAPPA
            0x004c: 0x039b, # GREEK CAPITAL LETTER LAMDA
            0x004d: 0x039c, # GREEK CAPITAL LETTER MU
            0x004e: 0x039d, # GREEK CAPITAL LETTER NU
            0x004f: 0x039f, # GREEK CAPITAL LETTER OMICRON
            0x0050: 0x03a0, # GREEK CAPITAL LETTER PI
            0x0051: 0x0398, # GREEK CAPITAL LETTER THETA
            0x0052: 0x03a1, # GREEK CAPITAL LETTER RHO
            0x0053: 0x03a3, # GREEK CAPITAL LETTER SIGMA
            0x0054: 0x03a4, # GREEK CAPITAL LETTER TAU
            0x0055: 0x03a5, # GREEK CAPITAL LETTER UPSILON
            0x0056: 0x03c2, # GREEK SMALL LETTER FINAL SIGMA
            0x0057: 0x2126, # OHM SIGN
            0x0058: 0x039e, # GREEK CAPITAL LETTER XI
            0x0059: 0x03a8, # GREEK CAPITAL LETTER PSI
            0x005a: 0x0396, # GREEK CAPITAL LETTER ZETA
            0x005c: 0x2234, # THEREFORE
            0x005e: 0x22a5, # UP TACK
            0x0060: 0xf8e5, # [unknown unicode name for radicalex]
            0x0061: 0x03b1, # GREEK SMALL LETTER ALPHA
            0x0062: 0x03b2, # GREEK SMALL LETTER BETA
            0x0063: 0x03c7, # GREEK SMALL LETTER CHI
            0x0064: 0x03b4, # GREEK SMALL LETTER DELTA
            0x0065: 0x03b5, # GREEK SMALL LETTER EPSILON
            0x0066: 0x03c6, # GREEK SMALL LETTER PHI
            0x0067: 0x03b3, # GREEK SMALL LETTER GAMMA
            0x0068: 0x03b7, # GREEK SMALL LETTER ETA
            0x0069: 0x03b9, # GREEK SMALL LETTER IOTA
            0x006a: 0x03d5, # GREEK PHI SYMBOL
            0x006b: 0x03ba, # GREEK SMALL LETTER KAPPA
            0x006c: 0x03bb, # GREEK SMALL LETTER LAMDA
            0x006d: 0x00b5, # MICRO SIGN
            0x006e: 0x03bd, # GREEK SMALL LETTER NU
            0x006f: 0x03bf, # GREEK SMALL LETTER OMICRON
            0x0070: 0x03c0, # GREEK SMALL LETTER PI
            0x0071: 0x03b8, # GREEK SMALL LETTER THETA
            0x0072: 0x03c1, # GREEK SMALL LETTER RHO
            0x0073: 0x03c3, # GREEK SMALL LETTER SIGMA
            0x0074: 0x03c4, # GREEK SMALL LETTER TAU
            0x0075: 0x03c5, # GREEK SMALL LETTER UPSILON
            0x0076: 0x03d6, # GREEK PI SYMBOL
            0x0077: 0x03c9, # GREEK SMALL LETTER OMEGA
            0x0078: 0x03be, # GREEK SMALL LETTER XI
            0x0079: 0x03c8, # GREEK SMALL LETTER PSI
            0x007a: 0x03b6, # GREEK SMALL LETTER ZETA
            0x007e: 0x223c, # TILDE OPERATOR
            0x007f: None, # UNDEFINED
            0x0080: None, # UNDEFINED
            0x0081: None, # UNDEFINED
            0x0082: None, # UNDEFINED
            0x0083: None, # UNDEFINED
            0x0084: None, # UNDEFINED
            0x0085: None, # UNDEFINED
            0x0086: None, # UNDEFINED
            0x0087: None, # UNDEFINED
            0x0088: None, # UNDEFINED
            0x0089: None, # UNDEFINED
            0x008a: None, # UNDEFINED
            0x008b: None, # UNDEFINED
            0x008c: None, # UNDEFINED
            0x008d: None, # UNDEFINED
            0x008e: None, # UNDEFINED
            0x008f: None, # UNDEFINED
            0x0090: None, # UNDEFINED
            0x0091: None, # UNDEFINED
            0x0092: None, # UNDEFINED
            0x0093: None, # UNDEFINED
            0x0094: None, # UNDEFINED
            0x0095: None, # UNDEFINED
            0x0096: None, # UNDEFINED
            0x0097: None, # UNDEFINED
            0x0098: None, # UNDEFINED
            0x0099: None, # UNDEFINED
            0x009a: None, # UNDEFINED
            0x009b: None, # UNDEFINED
            0x009c: None, # UNDEFINED
            0x009d: None, # UNDEFINED
            0x009e: None, # UNDEFINED
            0x009f: None, # UNDEFINED
            0x00a0: 0x20ac, # EURO SIGN
            0x00a1: 0x03d2, # GREEK UPSILON WITH HOOK SYMBOL
            0x00a2: 0x2032, # PRIME
            0x00a3: 0x2264, # LESS-THAN OR EQUAL TO
            0x00a4: 0x2044, # FRACTION SLASH
            0x00a5: 0x221e, # INFINITY
            0x00a6: 0x0192, # LATIN SMALL LETTER F WITH HOOK
            0x00a7: 0x2663, # BLACK CLUB SUIT
            0x00a8: 0x2666, # BLACK DIAMOND SUIT
            0x00a9: 0x2665, # BLACK HEART SUIT
            0x00aa: 0x2660, # BLACK SPADE SUIT
            0x00ab: 0x2194, # LEFT RIGHT ARROW
            0x00ac: 0x2190, # LEFTWARDS ARROW
            0x00ad: 0x2191, # UPWARDS ARROW
            0x00ae: 0x2192, # RIGHTWARDS ARROW
            0x00af: 0x2193, # DOWNWARDS ARROW
            0x00b2: 0x2033, # DOUBLE PRIME
            0x00b3: 0x2265, # GREATER-THAN OR EQUAL TO
            0x00b4: 0x00d7, # MULTIPLICATION SIGN
            0x00b5: 0x221d, # PROPORTIONAL TO
            0x00b6: 0x2202, # PARTIAL DIFFERENTIAL
            0x00b7: 0x2022, # BULLET
            0x00b8: 0x00f7, # DIVISION SIGN
            0x00b9: 0x2260, # NOT EQUAL TO
            0x00ba: 0x2261, # IDENTICAL TO
            0x00bb: 0x2248, # ALMOST EQUAL TO
            0x00bc: 0x2026, # HORIZONTAL ELLIPSIS
            0x00bd: 0xf8e6, # [unknown unicode name for arrowvertex]
            0x00be: 0xf8e7, # [unknown unicode name for arrowhorizex]
            0x00bf: 0x21b5, # DOWNWARDS ARROW WITH CORNER LEFTWARDS
            0x00c0: 0x2135, # ALEF SYMBOL
            0x00c1: 0x2111, # BLACK-LETTER CAPITAL I
            0x00c2: 0x211c, # BLACK-LETTER CAPITAL R
            0x00c3: 0x2118, # SCRIPT CAPITAL P
            0x00c4: 0x2297, # CIRCLED TIMES
            0x00c5: 0x2295, # CIRCLED PLUS
            0x00c6: 0x2205, # EMPTY SET
            0x00c7: 0x2229, # INTERSECTION
            0x00c8: 0x222a, # UNION
            0x00c9: 0x2283, # SUPERSET OF
            0x00ca: 0x2287, # SUPERSET OF OR EQUAL TO
            0x00cb: 0x2284, # NOT A SUBSET OF
            0x00cc: 0x2282, # SUBSET OF
            0x00cd: 0x2286, # SUBSET OF OR EQUAL TO
            0x00ce: 0x2208, # ELEMENT OF
            0x00cf: 0x2209, # NOT AN ELEMENT OF
            0x00d0: 0x2220, # ANGLE
            0x00d1: 0x2207, # NABLA
            0x00d2: 0xf6da, # [unknown unicode name for registerserif]
            0x00d3: 0xf6d9, # [unknown unicode name for copyrightserif]
            0x00d4: 0xf6db, # [unknown unicode name for trademarkserif]
            0x00d5: 0x220f, # N-ARY PRODUCT
            0x00d6: 0x221a, # SQUARE ROOT
            0x00d7: 0x22c5, # DOT OPERATOR
            0x00d8: 0x00ac, # NOT SIGN
            0x00d9: 0x2227, # LOGICAL AND
            0x00da: 0x2228, # LOGICAL OR
            0x00db: 0x21d4, # LEFT RIGHT DOUBLE ARROW
            0x00dc: 0x21d0, # LEFTWARDS DOUBLE ARROW
            0x00dd: 0x21d1, # UPWARDS DOUBLE ARROW
            0x00de: 0x21d2, # RIGHTWARDS DOUBLE ARROW
            0x00df: 0x21d3, # DOWNWARDS DOUBLE ARROW
            0x00e0: 0x25ca, # LOZENGE
            0x00e1: 0x2329, # LEFT-POINTING ANGLE BRACKET
            0x00e2: 0xf8e8, # [unknown unicode name for registersans]
            0x00e3: 0xf8e9, # [unknown unicode name for copyrightsans]
            0x00e4: 0xf8ea, # [unknown unicode name for trademarksans]
            0x00e5: 0x2211, # N-ARY SUMMATION
            0x00e6: 0xf8eb, # [unknown unicode name for parenlefttp]
            0x00e7: 0xf8ec, # [unknown unicode name for parenleftex]
            0x00e8: 0xf8ed, # [unknown unicode name for parenleftbt]
            0x00e9: 0xf8ee, # [unknown unicode name for bracketlefttp]
            0x00ea: 0xf8ef, # [unknown unicode name for bracketleftex]
            0x00eb: 0xf8f0, # [unknown unicode name for bracketleftbt]
            0x00ec: 0xf8f1, # [unknown unicode name for bracelefttp]
            0x00ed: 0xf8f2, # [unknown unicode name for braceleftmid]
            0x00ee: 0xf8f3, # [unknown unicode name for braceleftbt]
            0x00ef: 0xf8f4, # [unknown unicode name for braceex]
            0x00f0: None, # UNDEFINED
            0x00f1: 0x232a, # RIGHT-POINTING ANGLE BRACKET
            0x00f2: 0x222b, # INTEGRAL
            0x00f3: 0x2320, # TOP HALF INTEGRAL
            0x00f4: 0xf8f5, # [unknown unicode name for integralex]
            0x00f5: 0x2321, # BOTTOM HALF INTEGRAL
            0x00f6: 0xf8f6, # [unknown unicode name for parenrighttp]
            0x00f7: 0xf8f7, # [unknown unicode name for parenrightex]
            0x00f8: 0xf8f8, # [unknown unicode name for parenrightbt]
            0x00f9: 0xf8f9, # [unknown unicode name for bracketrighttp]
            0x00fa: 0xf8fa, # [unknown unicode name for bracketrightex]
            0x00fb: 0xf8fb, # [unknown unicode name for bracketrightbt]
            0x00fc: 0xf8fc, # [unknown unicode name for bracerighttp]
            0x00fd: 0xf8fd, # [unknown unicode name for bracerightmid]
            0x00fe: 0xf8fe, # [unknown unicode name for bracerightbt]
            0x00ff: None, # UNDEFINED
            },
            {
            0x0394:0x0044, # GREEK CAPITAL LETTER DELTA
            0x03a9:0x0057, # GREEK CAPITAL LETTER OMEGA
            0x03bc:0x006d, # GREEK SMALL LETTER MU
            }
            ),
    'zapfdingbats':StdCodecData({
            0x0021: 0x2701, # UPPER BLADE SCISSORS
            0x0022: 0x2702, # BLACK SCISSORS
            0x0023: 0x2703, # LOWER BLADE SCISSORS
            0x0024: 0x2704, # WHITE SCISSORS
            0x0025: 0x260e, # BLACK TELEPHONE
            0x0026: 0x2706, # TELEPHONE LOCATION SIGN
            0x0027: 0x2707, # TAPE DRIVE
            0x0028: 0x2708, # AIRPLANE
            0x0029: 0x2709, # ENVELOPE
            0x002a: 0x261b, # BLACK RIGHT POINTING INDEX
            0x002b: 0x261e, # WHITE RIGHT POINTING INDEX
            0x002c: 0x270c, # VICTORY HAND
            0x002d: 0x270d, # WRITING HAND
            0x002e: 0x270e, # LOWER RIGHT PENCIL
            0x002f: 0x270f, # PENCIL
            0x0030: 0x2710, # UPPER RIGHT PENCIL
            0x0031: 0x2711, # WHITE NIB
            0x0032: 0x2712, # BLACK NIB
            0x0033: 0x2713, # CHECK MARK
            0x0034: 0x2714, # HEAVY CHECK MARK
            0x0035: 0x2715, # MULTIPLICATION X
            0x0036: 0x2716, # HEAVY MULTIPLICATION X
            0x0037: 0x2717, # BALLOT X
            0x0038: 0x2718, # HEAVY BALLOT X
            0x0039: 0x2719, # OUTLINED GREEK CROSS
            0x003a: 0x271a, # HEAVY GREEK CROSS
            0x003b: 0x271b, # OPEN CENTRE CROSS
            0x003c: 0x271c, # HEAVY OPEN CENTRE CROSS
            0x003d: 0x271d, # LATIN CROSS
            0x003e: 0x271e, # SHADOWED WHITE LATIN CROSS
            0x003f: 0x271f, # OUTLINED LATIN CROSS
            0x0040: 0x2720, # MALTESE CROSS
            0x0041: 0x2721, # STAR OF DAVID
            0x0042: 0x2722, # FOUR TEARDROP-SPOKED ASTERISK
            0x0043: 0x2723, # FOUR BALLOON-SPOKED ASTERISK
            0x0044: 0x2724, # HEAVY FOUR BALLOON-SPOKED ASTERISK
            0x0045: 0x2725, # FOUR CLUB-SPOKED ASTERISK
            0x0046: 0x2726, # BLACK FOUR POINTED STAR
            0x0047: 0x2727, # WHITE FOUR POINTED STAR
            0x0048: 0x2605, # BLACK STAR
            0x0049: 0x2729, # STRESS OUTLINED WHITE STAR
            0x004a: 0x272a, # CIRCLED WHITE STAR
            0x004b: 0x272b, # OPEN CENTRE BLACK STAR
            0x004c: 0x272c, # BLACK CENTRE WHITE STAR
            0x004d: 0x272d, # OUTLINED BLACK STAR
            0x004e: 0x272e, # HEAVY OUTLINED BLACK STAR
            0x004f: 0x272f, # PINWHEEL STAR
            0x0050: 0x2730, # SHADOWED WHITE STAR
            0x0051: 0x2731, # HEAVY ASTERISK
            0x0052: 0x2732, # OPEN CENTRE ASTERISK
            0x0053: 0x2733, # EIGHT SPOKED ASTERISK
            0x0054: 0x2734, # EIGHT POINTED BLACK STAR
            0x0055: 0x2735, # EIGHT POINTED PINWHEEL STAR
            0x0056: 0x2736, # SIX POINTED BLACK STAR
            0x0057: 0x2737, # EIGHT POINTED RECTILINEAR BLACK STAR
            0x0058: 0x2738, # HEAVY EIGHT POINTED RECTILINEAR BLACK STAR
            0x0059: 0x2739, # TWELVE POINTED BLACK STAR
            0x005a: 0x273a, # SIXTEEN POINTED ASTERISK
            0x005b: 0x273b, # TEARDROP-SPOKED ASTERISK
            0x005c: 0x273c, # OPEN CENTRE TEARDROP-SPOKED ASTERISK
            0x005d: 0x273d, # HEAVY TEARDROP-SPOKED ASTERISK
            0x005e: 0x273e, # SIX PETALLED BLACK AND WHITE FLORETTE
            0x005f: 0x273f, # BLACK FLORETTE
            0x0060: 0x2740, # WHITE FLORETTE
            0x0061: 0x2741, # EIGHT PETALLED OUTLINED BLACK FLORETTE
            0x0062: 0x2742, # CIRCLED OPEN CENTRE EIGHT POINTED STAR
            0x0063: 0x2743, # HEAVY TEARDROP-SPOKED PINWHEEL ASTERISK
            0x0064: 0x2744, # SNOWFLAKE
            0x0065: 0x2745, # TIGHT TRIFOLIATE SNOWFLAKE
            0x0066: 0x2746, # HEAVY CHEVRON SNOWFLAKE
            0x0067: 0x2747, # SPARKLE
            0x0068: 0x2748, # HEAVY SPARKLE
            0x0069: 0x2749, # BALLOON-SPOKED ASTERISK
            0x006a: 0x274a, # EIGHT TEARDROP-SPOKED PROPELLER ASTERISK
            0x006b: 0x274b, # HEAVY EIGHT TEARDROP-SPOKED PROPELLER ASTERISK
            0x006c: 0x25cf, # BLACK CIRCLE
            0x006d: 0x274d, # SHADOWED WHITE CIRCLE
            0x006e: 0x25a0, # BLACK SQUARE
            0x006f: 0x274f, # LOWER RIGHT DROP-SHADOWED WHITE SQUARE
            0x0070: 0x2750, # UPPER RIGHT DROP-SHADOWED WHITE SQUARE
            0x0071: 0x2751, # LOWER RIGHT SHADOWED WHITE SQUARE
            0x0072: 0x2752, # UPPER RIGHT SHADOWED WHITE SQUARE
            0x0073: 0x25b2, # BLACK UP-POINTING TRIANGLE
            0x0074: 0x25bc, # BLACK DOWN-POINTING TRIANGLE
            0x0075: 0x25c6, # BLACK DIAMOND
            0x0076: 0x2756, # BLACK DIAMOND MINUS WHITE X
            0x0077: 0x25d7, # RIGHT HALF BLACK CIRCLE
            0x0078: 0x2758, # LIGHT VERTICAL BAR
            0x0079: 0x2759, # MEDIUM VERTICAL BAR
            0x007a: 0x275a, # HEAVY VERTICAL BAR
            0x007b: 0x275b, # HEAVY SINGLE TURNED COMMA QUOTATION MARK ORNAMENT
            0x007c: 0x275c, # HEAVY SINGLE COMMA QUOTATION MARK ORNAMENT
            0x007d: 0x275d, # HEAVY DOUBLE TURNED COMMA QUOTATION MARK ORNAMENT
            0x007e: 0x275e, # HEAVY DOUBLE COMMA QUOTATION MARK ORNAMENT
            0x007f: None, # UNDEFINED
            0x0080: 0x2768, # MEDIUM LEFT PARENTHESIS ORNAMENT
            0x0081: 0x2769, # MEDIUM RIGHT PARENTHESIS ORNAMENT
            0x0082: 0x276a, # MEDIUM FLATTENED LEFT PARENTHESIS ORNAMENT
            0x0083: 0x276b, # MEDIUM FLATTENED RIGHT PARENTHESIS ORNAMENT
            0x0084: 0x276c, # MEDIUM LEFT-POINTING ANGLE BRACKET ORNAMENT
            0x0085: 0x276d, # MEDIUM RIGHT-POINTING ANGLE BRACKET ORNAMENT
            0x0086: 0x276e, # HEAVY LEFT-POINTING ANGLE QUOTATION MARK ORNAMENT
            0x0087: 0x276f, # HEAVY RIGHT-POINTING ANGLE QUOTATION MARK ORNAMENT
            0x0088: 0x2770, # HEAVY LEFT-POINTING ANGLE BRACKET ORNAMENT
            0x0089: 0x2771, # HEAVY RIGHT-POINTING ANGLE BRACKET ORNAMENT
            0x008a: 0x2772, # LIGHT LEFT TORTOISE SHELL BRACKET ORNAMENT
            0x008b: 0x2773, # LIGHT RIGHT TORTOISE SHELL BRACKET ORNAMENT
            0x008c: 0x2774, # MEDIUM LEFT CURLY BRACKET ORNAMENT
            0x008d: 0x2775, # MEDIUM RIGHT CURLY BRACKET ORNAMENT
            0x008e: None, # UNDEFINED
            0x008f: None, # UNDEFINED
            0x0090: None, # UNDEFINED
            0x0091: None, # UNDEFINED
            0x0092: None, # UNDEFINED
            0x0093: None, # UNDEFINED
            0x0094: None, # UNDEFINED
            0x0095: None, # UNDEFINED
            0x0096: None, # UNDEFINED
            0x0097: None, # UNDEFINED
            0x0098: None, # UNDEFINED
            0x0099: None, # UNDEFINED
            0x009a: None, # UNDEFINED
            0x009b: None, # UNDEFINED
            0x009c: None, # UNDEFINED
            0x009d: None, # UNDEFINED
            0x009e: None, # UNDEFINED
            0x009f: None, # UNDEFINED
            0x00a0: None, # UNDEFINED
            0x00a1: 0x2761, # CURVED STEM PARAGRAPH SIGN ORNAMENT
            0x00a2: 0x2762, # HEAVY EXCLAMATION MARK ORNAMENT
            0x00a3: 0x2763, # HEAVY HEART EXCLAMATION MARK ORNAMENT
            0x00a4: 0x2764, # HEAVY BLACK HEART
            0x00a5: 0x2765, # ROTATED HEAVY BLACK HEART BULLET
            0x00a6: 0x2766, # FLORAL HEART
            0x00a7: 0x2767, # ROTATED FLORAL HEART BULLET
            0x00a8: 0x2663, # BLACK CLUB SUIT
            0x00a9: 0x2666, # BLACK DIAMOND SUIT
            0x00aa: 0x2665, # BLACK HEART SUIT
            0x00ab: 0x2660, # BLACK SPADE SUIT
            0x00ac: 0x2460, # CIRCLED DIGIT ONE
            0x00ad: 0x2461, # CIRCLED DIGIT TWO
            0x00ae: 0x2462, # CIRCLED DIGIT THREE
            0x00af: 0x2463, # CIRCLED DIGIT FOUR
            0x00b0: 0x2464, # CIRCLED DIGIT FIVE
            0x00b1: 0x2465, # CIRCLED DIGIT SIX
            0x00b2: 0x2466, # CIRCLED DIGIT SEVEN
            0x00b3: 0x2467, # CIRCLED DIGIT EIGHT
            0x00b4: 0x2468, # CIRCLED DIGIT NINE
            0x00b5: 0x2469, # CIRCLED NUMBER TEN
            0x00b6: 0x2776, # DINGBAT NEGATIVE CIRCLED DIGIT ONE
            0x00b7: 0x2777, # DINGBAT NEGATIVE CIRCLED DIGIT TWO
            0x00b8: 0x2778, # DINGBAT NEGATIVE CIRCLED DIGIT THREE
            0x00b9: 0x2779, # DINGBAT NEGATIVE CIRCLED DIGIT FOUR
            0x00ba: 0x277a, # DINGBAT NEGATIVE CIRCLED DIGIT FIVE
            0x00bb: 0x277b, # DINGBAT NEGATIVE CIRCLED DIGIT SIX
            0x00bc: 0x277c, # DINGBAT NEGATIVE CIRCLED DIGIT SEVEN
            0x00bd: 0x277d, # DINGBAT NEGATIVE CIRCLED DIGIT EIGHT
            0x00be: 0x277e, # DINGBAT NEGATIVE CIRCLED DIGIT NINE
            0x00bf: 0x277f, # DINGBAT NEGATIVE CIRCLED NUMBER TEN
            0x00c0: 0x2780, # DINGBAT CIRCLED SANS-SERIF DIGIT ONE
            0x00c1: 0x2781, # DINGBAT CIRCLED SANS-SERIF DIGIT TWO
            0x00c2: 0x2782, # DINGBAT CIRCLED SANS-SERIF DIGIT THREE
            0x00c3: 0x2783, # DINGBAT CIRCLED SANS-SERIF DIGIT FOUR
            0x00c4: 0x2784, # DINGBAT CIRCLED SANS-SERIF DIGIT FIVE
            0x00c5: 0x2785, # DINGBAT CIRCLED SANS-SERIF DIGIT SIX
            0x00c6: 0x2786, # DINGBAT CIRCLED SANS-SERIF DIGIT SEVEN
            0x00c7: 0x2787, # DINGBAT CIRCLED SANS-SERIF DIGIT EIGHT
            0x00c8: 0x2788, # DINGBAT CIRCLED SANS-SERIF DIGIT NINE
            0x00c9: 0x2789, # DINGBAT CIRCLED SANS-SERIF NUMBER TEN
            0x00ca: 0x278a, # DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT ONE
            0x00cb: 0x278b, # DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT TWO
            0x00cc: 0x278c, # DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT THREE
            0x00cd: 0x278d, # DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT FOUR
            0x00ce: 0x278e, # DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT FIVE
            0x00cf: 0x278f, # DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SIX
            0x00d0: 0x2790, # DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SEVEN
            0x00d1: 0x2791, # DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT EIGHT
            0x00d2: 0x2792, # DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT NINE
            0x00d3: 0x2793, # DINGBAT NEGATIVE CIRCLED SANS-SERIF NUMBER TEN
            0x00d4: 0x2794, # HEAVY WIDE-HEADED RIGHTWARDS ARROW
            0x00d5: 0x2192, # RIGHTWARDS ARROW
            0x00d6: 0x2194, # LEFT RIGHT ARROW
            0x00d7: 0x2195, # UP DOWN ARROW
            0x00d8: 0x2798, # HEAVY SOUTH EAST ARROW
            0x00d9: 0x2799, # HEAVY RIGHTWARDS ARROW
            0x00da: 0x279a, # HEAVY NORTH EAST ARROW
            0x00db: 0x279b, # DRAFTING POINT RIGHTWARDS ARROW
            0x00dc: 0x279c, # HEAVY ROUND-TIPPED RIGHTWARDS ARROW
            0x00dd: 0x279d, # TRIANGLE-HEADED RIGHTWARDS ARROW
            0x00de: 0x279e, # HEAVY TRIANGLE-HEADED RIGHTWARDS ARROW
            0x00df: 0x279f, # DASHED TRIANGLE-HEADED RIGHTWARDS ARROW
            0x00e0: 0x27a0, # HEAVY DASHED TRIANGLE-HEADED RIGHTWARDS ARROW
            0x00e1: 0x27a1, # BLACK RIGHTWARDS ARROW
            0x00e2: 0x27a2, # THREE-D TOP-LIGHTED RIGHTWARDS ARROWHEAD
            0x00e3: 0x27a3, # THREE-D BOTTOM-LIGHTED RIGHTWARDS ARROWHEAD
            0x00e4: 0x27a4, # BLACK RIGHTWARDS ARROWHEAD
            0x00e5: 0x27a5, # HEAVY BLACK CURVED DOWNWARDS AND RIGHTWARDS ARROW
            0x00e6: 0x27a6, # HEAVY BLACK CURVED UPWARDS AND RIGHTWARDS ARROW
            0x00e7: 0x27a7, # SQUAT BLACK RIGHTWARDS ARROW
            0x00e8: 0x27a8, # HEAVY CONCAVE-POINTED BLACK RIGHTWARDS ARROW
            0x00e9: 0x27a9, # RIGHT-SHADED WHITE RIGHTWARDS ARROW
            0x00ea: 0x27aa, # LEFT-SHADED WHITE RIGHTWARDS ARROW
            0x00eb: 0x27ab, # BACK-TILTED SHADOWED WHITE RIGHTWARDS ARROW
            0x00ec: 0x27ac, # FRONT-TILTED SHADOWED WHITE RIGHTWARDS ARROW
            0x00ed: 0x27ad, # HEAVY LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW
            0x00ee: 0x27ae, # HEAVY UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW
            0x00ef: 0x27af, # NOTCHED LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW
            0x00f0: None, # UNDEFINED
            0x00f1: 0x27b1, # NOTCHED UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW
            0x00f2: 0x27b2, # CIRCLED HEAVY WHITE RIGHTWARDS ARROW
            0x00f3: 0x27b3, # WHITE-FEATHERED RIGHTWARDS ARROW
            0x00f4: 0x27b4, # BLACK-FEATHERED SOUTH EAST ARROW
            0x00f5: 0x27b5, # BLACK-FEATHERED RIGHTWARDS ARROW
            0x00f6: 0x27b6, # BLACK-FEATHERED NORTH EAST ARROW
            0x00f7: 0x27b7, # HEAVY BLACK-FEATHERED SOUTH EAST ARROW
            0x00f8: 0x27b8, # HEAVY BLACK-FEATHERED RIGHTWARDS ARROW
            0x00f9: 0x27b9, # HEAVY BLACK-FEATHERED NORTH EAST ARROW
            0x00fa: 0x27ba, # TEARDROP-BARBED RIGHTWARDS ARROW
            0x00fb: 0x27bb, # HEAVY TEARDROP-SHANKED RIGHTWARDS ARROW
            0x00fc: 0x27bc, # WEDGE-TAILED RIGHTWARDS ARROW
            0x00fd: 0x27bd, # HEAVY WEDGE-TAILED RIGHTWARDS ARROW
            0x00fe: 0x27be, # OPEN-OUTLINED RIGHTWARDS ARROW
            0x00ff: None, # UNDEFINED
            },None),
    'pdfdoc':StdCodecData({
            0x007f: None, # UNDEFINED
            0x0080: 0x2022, # BULLET
            0x0081: 0x2020, # DAGGER
            0x0082: 0x2021, # DOUBLE DAGGER
            0x0083: 0x2026, # HORIZONTAL ELLIPSIS
            0x0084: 0x2014, # EM DASH
            0x0085: 0x2013, # EN DASH
            0x0086: 0x0192, # LATIN SMALL LETTER F WITH HOOK
            0x0087: 0x2044, # FRACTION SLASH
            0x0088: 0x2039, # SINGLE LEFT-POINTING ANGLE QUOTATION MARK
            0x0089: 0x203a, # SINGLE RIGHT-POINTING ANGLE QUOTATION MARK
            0x008a: 0x2212, # MINUS SIGN
            0x008b: 0x2030, # PER MILLE SIGN
            0x008c: 0x201e, # DOUBLE LOW-9 QUOTATION MARK
            0x008d: 0x201c, # LEFT DOUBLE QUOTATION MARK
            0x008e: 0x201d, # RIGHT DOUBLE QUOTATION MARK
            0x008f: 0x2018, # LEFT SINGLE QUOTATION MARK
            0x0090: 0x2019, # RIGHT SINGLE QUOTATION MARK
            0x0091: 0x201a, # SINGLE LOW-9 QUOTATION MARK
            0x0092: 0x2122, # TRADE MARK SIGN
            0x0093: 0xfb01, # LATIN SMALL LIGATURE FI
            0x0094: 0xfb02, # LATIN SMALL LIGATURE FL
            0x0095: 0x0141, # LATIN CAPITAL LETTER L WITH STROKE
            0x0096: 0x0152, # LATIN CAPITAL LIGATURE OE
            0x0097: 0x0160, # LATIN CAPITAL LETTER S WITH CARON
            0x0098: 0x0178, # LATIN CAPITAL LETTER Y WITH DIAERESIS
            0x0099: 0x017d, # LATIN CAPITAL LETTER Z WITH CARON
            0x009a: 0x0131, # LATIN SMALL LETTER DOTLESS I
            0x009b: 0x0142, # LATIN SMALL LETTER L WITH STROKE
            0x009c: 0x0153, # LATIN SMALL LIGATURE OE
            0x009d: 0x0161, # LATIN SMALL LETTER S WITH CARON
            0x009e: 0x017e, # LATIN SMALL LETTER Z WITH CARON
            0x009f: None, # UNDEFINED
            0x00a0: 0x20ac, # EURO SIGN
            0x00ad: None, # UNDEFINED
            24: 0x02d8, #breve
            25: 0x02c7, #caron
            26: 0x02c6, #circumflex
            27: 0x02d9, #dotaccent
            28: 0x02dd, #hungarumlaut
            29: 0x02db, #ogonek
            30: 0x02da, #ring
            31: 0x02dc, #tilde
            },None),
    'macexpert':StdCodecData({
            0x0021: 0xf721, # [unknown unicode name for exclamsmall]
            0x0022: 0xf6f8, # [unknown unicode name for Hungarumlautsmall]
            0x0023: 0xf7a2, # [unknown unicode name for centoldstyle]
            0x0024: 0xf724, # [unknown unicode name for dollaroldstyle]
            0x0025: 0xf6e4, # [unknown unicode name for dollarsuperior]
            0x0026: 0xf726, # [unknown unicode name for ampersandsmall]
            0x0027: 0xf7b4, # [unknown unicode name for Acutesmall]
            0x0028: 0x207d, # SUPERSCRIPT LEFT PARENTHESIS
            0x0029: 0x207e, # SUPERSCRIPT RIGHT PARENTHESIS
            0x002a: 0x2025, # TWO DOT LEADER
            0x002b: 0x2024, # ONE DOT LEADER
            0x002f: 0x2044, # FRACTION SLASH
            0x0030: 0xf730, # [unknown unicode name for zerooldstyle]
            0x0031: 0xf731, # [unknown unicode name for oneoldstyle]
            0x0032: 0xf732, # [unknown unicode name for twooldstyle]
            0x0033: 0xf733, # [unknown unicode name for threeoldstyle]
            0x0034: 0xf734, # [unknown unicode name for fouroldstyle]
            0x0035: 0xf735, # [unknown unicode name for fiveoldstyle]
            0x0036: 0xf736, # [unknown unicode name for sixoldstyle]
            0x0037: 0xf737, # [unknown unicode name for sevenoldstyle]
            0x0038: 0xf738, # [unknown unicode name for eightoldstyle]
            0x0039: 0xf739, # [unknown unicode name for nineoldstyle]
            0x003c: None, # UNDEFINED
            0x003d: 0xf6de, # [unknown unicode name for threequartersemdash]
            0x003e: None, # UNDEFINED
            0x003f: 0xf73f, # [unknown unicode name for questionsmall]
            0x0040: None, # UNDEFINED
            0x0041: None, # UNDEFINED
            0x0042: None, # UNDEFINED
            0x0043: None, # UNDEFINED
            0x0044: 0xf7f0, # [unknown unicode name for Ethsmall]
            0x0045: None, # UNDEFINED
            0x0046: None, # UNDEFINED
            0x0047: 0x00bc, # VULGAR FRACTION ONE QUARTER
            0x0048: 0x00bd, # VULGAR FRACTION ONE HALF
            0x0049: 0x00be, # VULGAR FRACTION THREE QUARTERS
            0x004a: 0x215b, # VULGAR FRACTION ONE EIGHTH
            0x004b: 0x215c, # VULGAR FRACTION THREE EIGHTHS
            0x004c: 0x215d, # VULGAR FRACTION FIVE EIGHTHS
            0x004d: 0x215e, # VULGAR FRACTION SEVEN EIGHTHS
            0x004e: 0x2153, # VULGAR FRACTION ONE THIRD
            0x004f: 0x2154, # VULGAR FRACTION TWO THIRDS
            0x0050: None, # UNDEFINED
            0x0051: None, # UNDEFINED
            0x0052: None, # UNDEFINED
            0x0053: None, # UNDEFINED
            0x0054: None, # UNDEFINED
            0x0055: None, # UNDEFINED
            0x0056: 0xfb00, # LATIN SMALL LIGATURE FF
            0x0057: 0xfb01, # LATIN SMALL LIGATURE FI
            0x0058: 0xfb02, # LATIN SMALL LIGATURE FL
            0x0059: 0xfb03, # LATIN SMALL LIGATURE FFI
            0x005a: 0xfb04, # LATIN SMALL LIGATURE FFL
            0x005b: 0x208d, # SUBSCRIPT LEFT PARENTHESIS
            0x005c: None, # UNDEFINED
            0x005d: 0x208e, # SUBSCRIPT RIGHT PARENTHESIS
            0x005e: 0xf6f6, # [unknown unicode name for Circumflexsmall]
            0x005f: 0xf6e5, # [unknown unicode name for hypheninferior]
            0x0060: 0xf760, # [unknown unicode name for Gravesmall]
            0x0061: 0xf761, # [unknown unicode name for Asmall]
            0x0062: 0xf762, # [unknown unicode name for Bsmall]
            0x0063: 0xf763, # [unknown unicode name for Csmall]
            0x0064: 0xf764, # [unknown unicode name for Dsmall]
            0x0065: 0xf765, # [unknown unicode name for Esmall]
            0x0066: 0xf766, # [unknown unicode name for Fsmall]
            0x0067: 0xf767, # [unknown unicode name for Gsmall]
            0x0068: 0xf768, # [unknown unicode name for Hsmall]
            0x0069: 0xf769, # [unknown unicode name for Ismall]
            0x006a: 0xf76a, # [unknown unicode name for Jsmall]
            0x006b: 0xf76b, # [unknown unicode name for Ksmall]
            0x006c: 0xf76c, # [unknown unicode name for Lsmall]
            0x006d: 0xf76d, # [unknown unicode name for Msmall]
            0x006e: 0xf76e, # [unknown unicode name for Nsmall]
            0x006f: 0xf76f, # [unknown unicode name for Osmall]
            0x0070: 0xf770, # [unknown unicode name for Psmall]
            0x0071: 0xf771, # [unknown unicode name for Qsmall]
            0x0072: 0xf772, # [unknown unicode name for Rsmall]
            0x0073: 0xf773, # [unknown unicode name for Ssmall]
            0x0074: 0xf774, # [unknown unicode name for Tsmall]
            0x0075: 0xf775, # [unknown unicode name for Usmall]
            0x0076: 0xf776, # [unknown unicode name for Vsmall]
            0x0077: 0xf777, # [unknown unicode name for Wsmall]
            0x0078: 0xf778, # [unknown unicode name for Xsmall]
            0x0079: 0xf779, # [unknown unicode name for Ysmall]
            0x007a: 0xf77a, # [unknown unicode name for Zsmall]
            0x007b: 0x20a1, # COLON SIGN
            0x007c: 0xf6dc, # [unknown unicode name for onefitted]
            0x007d: 0xf6dd, # [unknown unicode name for rupiah]
            0x007e: 0xf6fe, # [unknown unicode name for Tildesmall]
            0x007f: None, # UNDEFINED
            0x0080: None, # UNDEFINED
            0x0081: 0xf6e9, # [unknown unicode name for asuperior]
            0x0082: 0xf6e0, # [unknown unicode name for centsuperior]
            0x0083: None, # UNDEFINED
            0x0084: None, # UNDEFINED
            0x0085: None, # UNDEFINED
            0x0086: None, # UNDEFINED
            0x0087: 0xf7e1, # [unknown unicode name for Aacutesmall]
            0x0088: 0xf7e0, # [unknown unicode name for Agravesmall]
            0x0089: 0xf7e2, # [unknown unicode name for Acircumflexsmall]
            0x008a: 0xf7e4, # [unknown unicode name for Adieresissmall]
            0x008b: 0xf7e3, # [unknown unicode name for Atildesmall]
            0x008c: 0xf7e5, # [unknown unicode name for Aringsmall]
            0x008d: 0xf7e7, # [unknown unicode name for Ccedillasmall]
            0x008e: 0xf7e9, # [unknown unicode name for Eacutesmall]
            0x008f: 0xf7e8, # [unknown unicode name for Egravesmall]
            0x0090: 0xf7ea, # [unknown unicode name for Ecircumflexsmall]
            0x0091: 0xf7eb, # [unknown unicode name for Edieresissmall]
            0x0092: 0xf7ed, # [unknown unicode name for Iacutesmall]
            0x0093: 0xf7ec, # [unknown unicode name for Igravesmall]
            0x0094: 0xf7ee, # [unknown unicode name for Icircumflexsmall]
            0x0095: 0xf7ef, # [unknown unicode name for Idieresissmall]
            0x0096: 0xf7f1, # [unknown unicode name for Ntildesmall]
            0x0097: 0xf7f3, # [unknown unicode name for Oacutesmall]
            0x0098: 0xf7f2, # [unknown unicode name for Ogravesmall]
            0x0099: 0xf7f4, # [unknown unicode name for Ocircumflexsmall]
            0x009a: 0xf7f6, # [unknown unicode name for Odieresissmall]
            0x009b: 0xf7f5, # [unknown unicode name for Otildesmall]
            0x009c: 0xf7fa, # [unknown unicode name for Uacutesmall]
            0x009d: 0xf7f9, # [unknown unicode name for Ugravesmall]
            0x009e: 0xf7fb, # [unknown unicode name for Ucircumflexsmall]
            0x009f: 0xf7fc, # [unknown unicode name for Udieresissmall]
            0x00a0: None, # UNDEFINED
            0x00a1: 0x2078, # SUPERSCRIPT EIGHT
            0x00a2: 0x2084, # SUBSCRIPT FOUR
            0x00a3: 0x2083, # SUBSCRIPT THREE
            0x00a4: 0x2086, # SUBSCRIPT SIX
            0x00a5: 0x2088, # SUBSCRIPT EIGHT
            0x00a6: 0x2087, # SUBSCRIPT SEVEN
            0x00a7: 0xf6fd, # [unknown unicode name for Scaronsmall]
            0x00a8: None, # UNDEFINED
            0x00a9: 0xf6df, # [unknown unicode name for centinferior]
            0x00aa: 0x2082, # SUBSCRIPT TWO
            0x00ab: None, # UNDEFINED
            0x00ac: 0xf7a8, # [unknown unicode name for Dieresissmall]
            0x00ad: None, # UNDEFINED
            0x00ae: 0xf6f5, # [unknown unicode name for Caronsmall]
            0x00af: 0xf6f0, # [unknown unicode name for osuperior]
            0x00b0: 0x2085, # SUBSCRIPT FIVE
            0x00b1: None, # UNDEFINED
            0x00b2: 0xf6e1, # [unknown unicode name for commainferior]
            0x00b3: 0xf6e7, # [unknown unicode name for periodinferior]
            0x00b4: 0xf7fd, # [unknown unicode name for Yacutesmall]
            0x00b5: None, # UNDEFINED
            0x00b6: 0xf6e3, # [unknown unicode name for dollarinferior]
            0x00b7: None, # UNDEFINED
            0x00b8: None, # UNDEFINED
            0x00b9: 0xf7fe, # [unknown unicode name for Thornsmall]
            0x00ba: None, # UNDEFINED
            0x00bb: 0x2089, # SUBSCRIPT NINE
            0x00bc: 0x2080, # SUBSCRIPT ZERO
            0x00bd: 0xf6ff, # [unknown unicode name for Zcaronsmall]
            0x00be: 0xf7e6, # [unknown unicode name for AEsmall]
            0x00bf: 0xf7f8, # [unknown unicode name for Oslashsmall]
            0x00c0: 0xf7bf, # [unknown unicode name for questiondownsmall]
            0x00c1: 0x2081, # SUBSCRIPT ONE
            0x00c2: 0xf6f9, # [unknown unicode name for Lslashsmall]
            0x00c3: None, # UNDEFINED
            0x00c4: None, # UNDEFINED
            0x00c5: None, # UNDEFINED
            0x00c6: None, # UNDEFINED
            0x00c7: None, # UNDEFINED
            0x00c8: None, # UNDEFINED
            0x00c9: 0xf7b8, # [unknown unicode name for Cedillasmall]
            0x00ca: None, # UNDEFINED
            0x00cb: None, # UNDEFINED
            0x00cc: None, # UNDEFINED
            0x00cd: None, # UNDEFINED
            0x00ce: None, # UNDEFINED
            0x00cf: 0xf6fa, # [unknown unicode name for OEsmall]
            0x00d0: 0x2012, # FIGURE DASH
            0x00d1: 0xf6e6, # [unknown unicode name for hyphensuperior]
            0x00d2: None, # UNDEFINED
            0x00d3: None, # UNDEFINED
            0x00d4: None, # UNDEFINED
            0x00d5: None, # UNDEFINED
            0x00d6: 0xf7a1, # [unknown unicode name for exclamdownsmall]
            0x00d7: None, # UNDEFINED
            0x00d8: 0xf7ff, # [unknown unicode name for Ydieresissmall]
            0x00d9: None, # UNDEFINED
            0x00da: 0x00b9, # SUPERSCRIPT ONE
            0x00db: 0x00b2, # SUPERSCRIPT TWO
            0x00dc: 0x00b3, # SUPERSCRIPT THREE
            0x00dd: 0x2074, # SUPERSCRIPT FOUR
            0x00de: 0x2075, # SUPERSCRIPT FIVE
            0x00df: 0x2076, # SUPERSCRIPT SIX
            0x00e0: 0x2077, # SUPERSCRIPT SEVEN
            0x00e1: 0x2079, # SUPERSCRIPT NINE
            0x00e2: 0x2070, # SUPERSCRIPT ZERO
            0x00e3: None, # UNDEFINED
            0x00e4: 0xf6ec, # [unknown unicode name for esuperior]
            0x00e5: 0xf6f1, # [unknown unicode name for rsuperior]
            0x00e6: 0xf6f3, # [unknown unicode name for tsuperior]
            0x00e7: None, # UNDEFINED
            0x00e8: None, # UNDEFINED
            0x00e9: 0xf6ed, # [unknown unicode name for isuperior]
            0x00ea: 0xf6f2, # [unknown unicode name for ssuperior]
            0x00eb: 0xf6eb, # [unknown unicode name for dsuperior]
            0x00ec: None, # UNDEFINED
            0x00ed: None, # UNDEFINED
            0x00ee: None, # UNDEFINED
            0x00ef: None, # UNDEFINED
            0x00f0: None, # UNDEFINED
            0x00f1: 0xf6ee, # [unknown unicode name for lsuperior]
            0x00f2: 0xf6fb, # [unknown unicode name for Ogoneksmall]
            0x00f3: 0xf6f4, # [unknown unicode name for Brevesmall]
            0x00f4: 0xf7af, # [unknown unicode name for Macronsmall]
            0x00f5: 0xf6ea, # [unknown unicode name for bsuperior]
            0x00f6: 0x207f, # SUPERSCRIPT LATIN SMALL LETTER N
            0x00f7: 0xf6ef, # [unknown unicode name for msuperior]
            0x00f8: 0xf6e2, # [unknown unicode name for commasuperior]
            0x00f9: 0xf6e8, # [unknown unicode name for periodsuperior]
            0x00fa: 0xf6f7, # [unknown unicode name for Dotaccentsmall]
            0x00fb: 0xf6fc, # [unknown unicode name for Ringsmall]
            0x00fc: None, # UNDEFINED
            0x00fd: None, # UNDEFINED
            0x00fe: None, # UNDEFINED
            0x00ff: None, # UNDEFINED
            },None),
        }
    __rl_extension_codecs = {
            'extpdfdoc':ExtCodecData('pdfdoc',{0x000a:0x000a,0x000d:0x000d},None),
            }
    #for k,v in __rl_codecs_data.items():
    #   __rl_codecs_data[k+'enc'] = __rl_codecs_data[k+'encoding'] = v
    #del k,v

    __rl_dynamic_codecs = []

    def __init__(self):
        raise NotImplementedError

    @staticmethod
    def _makeCodecInfo(name,encoding_map,decoding_map):
        ### Codec APIs
        class Codec(codecs.Codec):
            def encode(self,input,errors='strict',charmap_encode=codecs.charmap_encode,encoding_map=encoding_map):
                return charmap_encode(input,errors,encoding_map)

            def decode(self,input,errors='strict',charmap_decode=codecs.charmap_decode,decoding_map=decoding_map):
                return charmap_decode(input,errors,decoding_map)

        class StreamWriter(Codec,codecs.StreamWriter):
            pass

        class StreamReader(Codec,codecs.StreamReader):
            pass
        C = Codec()
        return codecs.CodecInfo(C.encode,C.decode,streamreader=StreamReader,streamwriter=StreamWriter,name=name)

    @staticmethod
    def _256_exception_codec(name,exceptions,rexceptions,baseRange=range(32,256)):
        decoding_map = codecs.make_identity_dict(baseRange)
        decoding_map.update(exceptions)
        encoding_map = codecs.make_encoding_map(decoding_map)
        if rexceptions: encoding_map.update(rexceptions)
        return RL_Codecs._makeCodecInfo(name,encoding_map,decoding_map)

    __rl_codecs_cache = {}

    @staticmethod
    def __rl_codecs(name,cache=__rl_codecs_cache,data=__rl_codecs_data,extension_codecs=__rl_extension_codecs,_256=True):
        try:
            return cache[name]
        except KeyError:
            if name in extension_codecs:
                x = extension_codecs[name]
                e,r = data[x.baseName]
                if x.exceptions:
                    if e:
                        e = e.copy()
                        e.update(x.exceptions)
                    else:
                        e = x.exceptions
                if x.rexceptions:
                    if r:
                        r = r.copy()
                        r.update(x.rexceptions)
                    else:
                        r = x.exceptions
            else:
                e,r = data[name]
            cache[name] = c = RL_Codecs._256_exception_codec(name,e,r) if _256 else RL_Codecs._makeCodecInfo(name, e,r or {})
        return c

    @staticmethod
    def _rl_codecs(name):
        name = name.lower()
        from reportlab.pdfbase.pdfmetrics import standardEncodings
        for e in standardEncodings+('ExtPdfdocEncoding',):
            e = e[:-8].lower()
            if name.startswith(e): return RL_Codecs.__rl_codecs(e)
        if name in RL_Codecs.__rl_dynamic_codecs:
            return RL_Codecs.__rl_codecs(name,_256=False)
        return None

    @staticmethod
    def register():
        codecs.register(RL_Codecs._rl_codecs)

    @staticmethod
    def add_dynamic_codec(name,exceptions,rexceptions):
        name = name.lower()
        RL_Codecs.remove_dynamic_codec(name)
        RL_Codecs.__rl_codecs_data[name] = (exceptions,rexceptions)
        RL_Codecs.__rl_dynamic_codecs.append(name)

    @staticmethod
    def remove_dynamic_codec(name):
        name = name.lower()
        if name in RL_Codecs.__rl_dynamic_codecs:
            RL_Codecs.__rl_codecs_data.pop(name,None)
            RL_Codecs.__rl_codecs_cache.pop(name,None)
            RL_Codecs.__rl_dynamic_codecs.remove(name)

    @staticmethod
    def reset_dynamic_codecs():
        map(RL_Codecs.remove_dynamic_codec, RL_Codecs.__rl_dynamic_codecs)
